package kerberos.exceptions;

import java.net.Socket;

import kerberos.messages.M_ERROR;

import util.NetUtil;

/**
 * <p>Diese Exception bedeutet, dass ein bertragenes Array zu lang war.</p>
 * <p>In gewissem Sinne ist diese Exception sowohl eine KerberosException 
 * als auch eine IOException. Da der Grund aber bei Kerberos liegt, erbt sie von der 
 * KerberosException.</p>
 * @author Stephan Koops
 * @see kerberos.exceptions.SendeArrayZuLangException
 */
public abstract class ArrayZuLangException extends KerberosException
{
	private int lnge = -1; // negative Zahl bedeutet nicht gesetzt
	private int maxLnge = -1;
	
	/**
	 * @param lnge tatschliche Lnge
	 * @param maxLnge Maximal erlaubte Lnge
	 */
	ArrayZuLangException(int lnge, int maxLnge)
	{
		super("Das Array war "+lnge+" Zeichen lang, drfte aber nur "+maxLnge+" Zeichen lang sein");
	}
	
	/**
	 * @param lnge tatschliche Lnge
	 * @param maxLnge maximal erlaubte Lnge
	 */
	ArrayZuLangException(int lnge)
	{
		super("Das Array war "+lnge+" Zeichen lang, das ist zu lang");
	}

	ArrayZuLangException(String message)
	{
		super(message);
	}

	/**
	 * @return Die aus dem Netz angegebene Lnge des zu empfangenen Arrays
	 * @throws NullPointerException Wird geworfen, wenn die Lnge unbekannt ist 
	 */
	public int getLnge() throws NullPointerException
	{
		if (this.lnge < 0)
			throw new NullPointerException("Die Lnge ist ungesetzt");
		return lnge;
	}

	/**
	 * @return Die an dieser Stelle von der Software vorgegebene Maximallnge des Arrays
	 * @throws NullPointerException Wird geworfen, wenn die Maximallnge unbekannt ist 
	 */
	public int getMaxLnge()
	{
		if (this.maxLnge < 0)
			throw new NullPointerException("Die Maximallnge ist ungesetzt");
		return maxLnge;
	}

	/**
	 * Sendet eine Nachricht, dass die letzte Nachricht zu lang war durch den Socket.
	 * @param socket
	 * @param messageTyp
	 */
	public static void sendError(Socket socket, int messageTyp)
	{
		NetUtil.sendError(socket, messageTyp, M_ERROR.ERR_ARRAY_TO_LONG);
	}

	/**
	 * Sendet eine Nachricht, dass die letzte Nachricht zu lang war durch den Socket.
	 * Versendet die Nachricht mit dem Message-Typ unbekannt
	 * @see #sendError(Socket, int)
	 * @param socket
	 */
	public static void sendError(Socket socket)
	{
		sendError(socket, M_ERROR.M_TYP_UNBEKANNT);
	}
}
