/*
 * Created on 16.12.2003
 */
package kerberos.messages;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.packets.PacketException;

/**
 * @author Stephan
 */
public interface IMessageBuilder
{
	/**
	 *  Die einzelnen Nachrichten-Methoden mssen noch um Exceptions erweitert werden
	 *  und evtl. noch ergnzt oder abgendert werden.
	 * 
	 * @param benutzername Name des Benutzers
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildKdcAuthLogin(String benutzername) throws PacketException;
	
	/**
	 * 
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @param benutzername Name des Benutzers
	 * @param schlssel_TGS Schlssel des Ticket Granting Service 
	 * @param kdc Name des KDC
	 * @param startzeit Beginn der Gltigkeitsdauer des Tickets
	 * @param endzeit Ende der Gltigkeitsdauer des Tickets
	 * @param schlssel_C Schlssel des Clients
	 * @param schlssel_C_TGS Sitzungsschlssel zwischen Client und TGS
	 * @return byte[]
	 */
	public byte[] buildKdcAuthLoginOK(long nonce, String benutzername, byte[] schlssel_TGS, String kdc,
		long startzeit, long endzeit, byte[] schlssel_C, byte[] schlssel_C_TGS)
	throws PacketException;

	/**
	 * 
	 * @param ticketTGS Ticket fr den TGS
	 * @param benutzername Name des Benutzers
	 * @param schlssel_C_TGS Sitzungsschlssel zwischen dem Client und dem TGS
	 * @param zieldienstname  ehemals servername gewesen
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildTGSLogin(byte[] ticketTGS, String benutzername, byte[] schlssel_C_TGS,
		String zieldienstname) throws PacketException;

	/**
	 * 
	 * @param schluessel_CS
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @param benutzername Name des Benutzers
	 * @param servername
	 * @param startzeit
	 * @param endzeit
	 * @param schluessel_S
	 * @param schluessel_C_TGS
	 * @return byte[]
	 */
	public byte[] buildTGSLoginOK(byte[] schluessel_CS, long nonce, String benutzername, String servername,
		long startzeit, long endzeit, byte[] schluessel_S, byte[] schluessel_C_TGS) throws PacketException;

	/**
	 * Vom Server zum Client<br>
	 * Message von Application zum Client
	 * mussten hier noch einen Schlssel in die bergabe einfgen 
	 * @param daten
	 * @param schluessel
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildData(byte[] daten, byte[] schluessel) throws PacketException;
	
	/**
	 * @param daten Zu verschlsselnde Daten
	 * @param schluessel Schlssel zum verschlsseln
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @return byte[]
	 */
	public byte[] buildData(byte[] daten, byte[] schluessel, long nonce) throws PacketException;

	/**
	 * 
	 * @param serverticket Ticket fr den Zielserver
	 * @param benutzername Name des Benutzers
	 * @param schluessel_CS Sitzungsschlssel zwischen Client und Server
	 * @return Mess_mem_Nonce
	 * @throws PacketException
	 */
	public Mess_mem_Nonce buildApLogin(byte[] serverticket, String benutzername, byte[] schluessel_CS) throws PacketException;
	
	/**
	 * Vom Client zum Server<br>
	 * @param serverticket Ticket an den Server
	 * @param benutzername Name des Benutzers
	 * @param schluessel_CS Sitzungsschlssel zwischen Client und Zielserver
	 * @param daten ehemals Command
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildApLogin(byte[] serverticket, String benutzername, byte[] schluessel_CS,
		byte[] daten) throws PacketException;

	/**
	 * Message wenn Fehler aufgetreten ist.<br>
	 * Errorcodes siehe M_ERROR.java.
	 * @param messageType Typ der Nachricht bei der der Fehler auftrat
	 * @param errorcode Art des Fehlers
	 * @return byte[]
	 */	
	public byte[] buildError(int messageType, int errorcode) throws PacketException;

	/**
	 * @param message zu entschluesselnde Nachricht
	 * @param schluessel Schlssel zum entschlsseln
	 * @return Message
	 */	
	public Message readMessage(byte[] message, byte[] schluessel)
		throws PasswortFalschException, PacketException, ClientnameFalschException, UngltigerMessageTypException;
}
