/*
 * Created on Nov 3, 2003
 */
package kerberos.messages;

import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.packets.PacketException;
import kerberos.packets.PacketReader;

/**
 * Antwort des Applicationservers an den Client;
 * 
 * @author stier_j
 */

public class M_AP_REP extends Message implements M_AP
{
	
	byte[] daten = null;
	long nonce = 0;
	
	M_AP_REP(byte[] kryptDaten, byte[] Schluessel) throws PacketException, PasswortFalschException
	{
		super();
		PacketReader pr = new PacketReader(kryptDaten);
		daten = pr.readData(Schluessel);
		nonce = pr.readNonceNonCrypt();
	}
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.
	 * @throws NonceFalschException wenn die Noncewerte nicht bereinstimmen.
	 */	
	public void checkNonce(long mynonce) throws NonceFalschException
	{
		if(mynonce != nonce) throw new NonceFalschException();
	}
	
	/**
	 * Gibt die unverschlsselten Daten zurck.
	 * @return Daten.
	 */
	public byte[] getDaten()
	{
		return daten;
	}
	
	/**
	 * Gibt den ausgelesenen Noncewert zurck. Besser checkNonce() verwenden.
	 * @return Nonce.
	 */
	public long getNonce(){ return nonce; }

	/**
	 * @see kerberos.messages.Message#getType()
	 */
	public int getType()
	{
		return M_AP_REP;
	}
}
