/*
 * Created on Nov 3, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kerberos.messages;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.packets.*;

/**
 * Dient zum Bearbeiten der ersten Anfrage des Clients an den Applicationserver. 
 * @author stier_j
 */
public class M_AP_REQ extends Message implements M_AP, MessageWithTime
{
	
	long Nonce, startzeit, endzeit, zeit  = 0;
	byte[] ticket, AuthClient, Command, Schluessel_C_S = null;
	String Client1, Client2, Server = null;

	M_AP_REQ(byte[] Daten, byte[] Schluessel) throws PacketException,
		PasswortFalschException, ClientnameFalschException
	{
		super();
		PacketReader pr = new PacketReader(Daten);
		ticket = pr.readDataNonCrypt();
		PacketReader pr2 = new PacketReader(ticket);
		ticket = pr2.readData(Schluessel);
			
			PacketReader prt = new PacketReader(ticket);
			Client1 = prt.readStringNonCrypt();
			Server = prt.readStringNonCrypt();
			startzeit = prt.readLongNonCrypt();
			endzeit = prt.readLongNonCrypt();
			Schluessel_C_S = prt.readDataNonCrypt();
			
		AuthClient = pr.readDataNonCrypt();
		pr2 = new PacketReader(AuthClient);
		AuthClient = pr2.readData(Schluessel_C_S);
			
			prt = new PacketReader(AuthClient);
			Client2 = prt.readStringNonCrypt();
			zeit = prt.readLongNonCrypt();
			
		Command = pr.readData(Schluessel_C_S);
		Nonce = pr.readNonceNonCrypt();
			
		if(!Client1.equals(Client2))
		{
			throw new ClientnameFalschException(Client1, Client2);
		} 	
	}
	
	/**
	 * Clientname aus dem Ticket -
	 * wird wohl nach aussen hin nicht bentigt
	 * @return Client Name aus Ticket
	 */
	public String getClientNameTicket()
	{
		if(Client1 == null)
			throw new NullPointerException("Client1 ist null");
		return Client1;
	}
	
	/**
	 * Gibt den Usernamen zurck
	 * @return User Name aus berprfung
	 */
	public String getUsername()
	{
		if(Client2 == null)
			throw new NullPointerException("Client2 ist null");
		return Client2;
	}
	
	/**
	 * berprfen ob der eigene Server gemeint ist.
	 * @return ServerNamen
	 */
	public String getServerName()
	{
		if(Server == null)
			throw new NullPointerException("Server ist null");
		return Server;
	}
	
	/**
	 * Mit getEndZeit() vergleichen, ob Ticket noch im Zeitfenster ist.
	 * @return Gltigkeitsbeginn des Tickets.
	 */
	public long getStartzeit() { return startzeit; }
	
	/**
	 * Mit getStartZeit(), vergleichen ob Ticket noch im Zeitfenster ist.
	 * @return Ablaufzeit des Tickets.
	 */
	public long getEndzeit() { return endzeit; }
	
	/**
	 * Sitzungsschlssel zwischen Client und dem Applicationserver.
	 * @return Sitzungsschlssel.
	 */
	public byte[] getSchluessel_C_S()
	{
		if(Schluessel_C_S == null)
			throw new NullPointerException("Schluessel_C_S ist null");
		return Schluessel_C_S;
	}
	
	/**
	 * Aktueller Zeitpunkt um zu sehen, wann der Client geschickt hat.
	 * @return Zeit.
	 */
	public long getZeit() { return zeit; }
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.
	 * </br>
	 * <b>Hier nicht benutzt.</b>
	 */	
	public void checkNonce(long nonce) {}
	
	/**
	 * Gibt den ausgelesenen Noncewert zurck.
	 * @return Noncewert.
	 */
	public long getNonce(){ return Nonce; }
	
	/**
	 * Gibt das gesamte Ticket zurck.
	 * Wird eigentlich nicht gebraucht.
	 * @return ticket.
	 */
	public byte[] getTicket()
	{
		// TODO Die getTicket() Methode wird zur Zeit nicht benutzt
		if(ticket == null)
			throw new NullPointerException("ticket ist null");
		return ticket;
	}
	
	/* *
	 * Gibt die Authentifikation des Clients zurck.
	 * Wird nicht gebraucht.
	 * @return AuthClient.
	 * /
	private byte[] getAuthClient()
	{
		if(AuthClient == null)
			throw new NullPointerException("AuthClient ist null");
		return AuthClient;
	} //*/
	
	/**
	 * Gibt die Nutzdaten unverschlsselt zurck.
	 * @return Command.
	 */
	public byte[] getDaten()
	{
		if(Command == null)
			throw new NullPointerException("Command ist null");
		return Command;
	}

	/**
	 * @see kerberos.messages.MessageWithTime#getType()
	 */
	public int getType()
	{
		return M_AP_REQ;
	}
}
