/*
 * Created on Nov 3, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kerberos.messages;

import kerberos.exceptions.SendeArrayZuLangException;
import kerberos.exceptions.ServerdienstNichtVorhandenException;
import kerberos.exceptions.TicketAbgelaufenException;
import kerberos.exceptions.TicketNieGueltigException;
import kerberos.exceptions.TicketNochNichtGueltigException;
import kerberos.exceptions.TicketUngueltigException;
import kerberos.exceptions.UnbekannterParticipalException;
import kerberos.packets.PacketException;
import kerberos.packets.PacketReader;

/**
 * Dient zum Auslesen einer Error-Message.<br>
 * Konstanten fr den Errorcode (siehe RFC 1510 S.84/85):
 * @author stier_j
 */
public class M_ERROR extends Message 
{
	/**
	 * ERR_MSG_TYPE = anderer Nachrichtentyp als erwartet
	 */
	public static final int ERR_MSG_TYPE			=  0;
	/**
	 * ERR_TKT_EXPIRED = Ticket ist abgelaufen
	 */
	public static final int ERR_TKT_EXPIRED			=  1;
	/**
	 * ERR_REPEAT = Nachricht ist schon mal empfangen worden (Wiedereinspielung)
	 */
	public static final int ERR_REPEAT				=  2;
	/**
	 * ERR_BADMATCH = berprfung Ticket und Authentifikator fehlgeschlagen
	 */
	public static final int ERR_BADMATCH			=  3;
	/**
	 * ERR_C_PRINCIPAL_UNKNOWN = Unbekannter Client
	 */
	public static final int ERR_C_PRINCIPAL_UNKNOWN	=  4;
	/**
	 * ERR_S_PRINCIPAL_UNKNOWN = Unbekannter Server
	 */
	public static final int ERR_S_PRINCIPAL_UNKNOWN	=  5;
	/**
	 * ERR_NULL_KEY = Schlssel nicht vorhanden
	 */
	public static final int ERR_NULL_KEY			=  6;
	/**
	 * ERR_NEVER_VALID = Anfangszeit ist spter als Endzeit
	 */
	public static final int ERR_NEVER_VALID			=  7;
	/**
	 * ERR_TKT_NYV = Die Startzeit des Tickets ist noch nicht erreicht
	 */
	public static final int ERR_TKT_NYV				=  8;
	/**
	 * ERR_BADADDR = Falsche Adresse
	 */
	public static final int ERR_BADADDR				=  9;
	/**
	 * ERR_MODIFIED = Fehler im Nachrichtenstrom
	 */
	public static final int ERR_MODIFIED			= 10;
	
	/**
	 * ERR_ARRAY_TO_LONG = Es wurde ein unakzeptabel langes Array bertragen.
	 */
	public static final int ERR_ARRAY_TO_LONG       = 11;
	
	long messageType=0 , errorcode = 0;
	
	/**
	 * Die Fehlermeldungen werden unverschlsselt bertragen
	 * @param daten
	 * @throws PacketException
	 */
	M_ERROR(byte[] daten) throws PacketException
	{
		PacketReader pr = new PacketReader(daten);
		this.messageType = pr.readLongNonCrypt();
		this.errorcode = pr.readLongNonCrypt();
	}
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.<br>
	 * <b>Hier nicht benutzt.</b>
	 */	
	public void checkNonce(long nonce)
	{
	}
	
	/**
	 * Gibt den Typ der Message auf die der Fehler erzeugt wurde zurck.
	 * @return messagetyp.
	 */
	public long getMessageType()
	{
		return messageType;
	}
	
	/**
	 * Gibt den Fehler Code der Message zurck.
	 * @return ErrorCode.
	 */
	public long getErrorcode()
	{
		return errorcode; 
	}
	
	public String toString()
	{
		return "Fehler-Message: MessageType: "+getString((int)this.getMessageType())+" Errorcode: "+this.getErrorcode();
	}
	
	/**
	 * Testet, ob diese Fehlermeldung bedeutet, dass das bergebene Ticket ungltig ist:<br>
	 * - Ticket von der Zeit her gltig<br>
	 * - Authentifikation OK? (noch nicht implementiert)
	 * @throws TicketUngueltigException
	 */
	public void checkForReasonInvalidTicket() throws TicketUngueltigException
	{
		if(this.errorcode == ERR_NEVER_VALID)
		{
			throw new TicketNieGueltigException();
		}
		if(this.errorcode == ERR_TKT_EXPIRED)
		{
			throw new TicketAbgelaufenException();
		}
		if(this.errorcode == ERR_TKT_NYV)
		{
			throw new TicketNochNichtGueltigException();
		}
		if(this.errorcode == ERR_BADMATCH)
		{
			// Ticket und Authentifikator fehlgeschlagen
		}
	}

	/**
	 * Testet, ob diese Fehlernachricht heit, dass der angegebene Principal
	 * (Name des Clients; Benutzer oder anderer Server) nicht existiert, und wirft 
	 * in dem Fall einen entsprechende Exception.
	 */
	public void checkForReasonInvalidPrincipal() throws UnbekannterParticipalException
	{
		if(this.errorcode == ERR_C_PRINCIPAL_UNKNOWN)
		{
			throw new UnbekannterParticipalException();
		}
	}

	/**
	 * Testet, ob diese Fehlernachricht heit, dass der gewnschte Zieldienst
	 * nicht existiert, und wirft in dem Fall einen entsprechende Exception.
	 */
	public void checkForReasonInvalidZieldienst() throws ServerdienstNichtVorhandenException
	{
		if(this.errorcode == ERR_S_PRINCIPAL_UNKNOWN)
		{
			throw new ServerdienstNichtVorhandenException();
		}
	}
	
	/**
	 * Testet, ob diese Fehlernachricht heit, ein zu langes Array bertragen wurde, 
	 * und wirft in dem Fall einen entsprechende Exception.
	 */
	public void checkForReasonArrayZuLang() throws SendeArrayZuLangException
	{
		if(this.errorcode == ERR_ARRAY_TO_LONG)
		{
			throw new SendeArrayZuLangException();
		}
	}
	
	/**
	 * Stub zum Testen, was diese Fehlernachricht aussagen 
	 * macht noch nichts produktives
	 */
	protected void x()
	{
		if(this.errorcode == ERR_BADADDR)
		{
			//throw new Falsche Adresse
		}
		if(this.errorcode == ERR_MODIFIED)
		{
			// Fehler im Nachrichtenstrom -> IOException
		}
		if(this.errorcode == ERR_MSG_TYPE)
		{
			// anderer Nachrichtentyp als erwartet
		}
		if(this.errorcode == ERR_NULL_KEY)
		{
			// Schlssel nicht vorhanden
		}
		if(this.errorcode == ERR_REPEAT)
		{
			// Nachricht schonmal eingespielt
		}
	}

	/**
	 * @see kerberos.messages.Message#getType()
	 */
	public int getType()
	{
		return M_ERROR;
	}
}
