/*
 * Created on Nov 3, 2003
 */
package kerberos.messages;

import kerberos.packets.*;
import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;

/**
 * Dient zum Auslesen der Antwort des Authenticationservices an den Client.
 * 
 * @author stier_j
 */
public class M_KDC_AS_REP extends Message {
	
	byte[] ticket = null, Schluessel_C_TGS = null, daten = null;
	long Nonce = 0;
	
	M_KDC_AS_REP(byte[] Daten, byte[] Schluessel) throws PacketException, PasswortFalschException
	{
		PacketReader pr = new PacketReader(Daten);
		daten = pr.readData(Schluessel);
			
		pr = new PacketReader(daten);
		Schluessel_C_TGS = pr.readDataNonCrypt();
		Nonce = pr.readNonceNonCrypt();
		ticket = pr.readDataNonCrypt();
	}
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.
	 * @throws NonceFalschException wenn die Noncewerte nicht bereinstimmen.
	 */	
	public void checkNonce(long mynonce) throws NonceFalschException
	{
		if(mynonce != Nonce) throw new NonceFalschException();
	}
	
	/**
	 * Wird fr M_KDC_TGS_REQ gebraucht. 
	 * @return TGS-Ticket.
	 */
	public byte[] getTicket()
	{
		if(ticket == null)
		{
			return new byte[0];
			//throw new NullPointerException();
		} 
		return ticket; 
	}
	
	/**
	 * Gibt den Sitzungschlssel zwischen dem Client und dem TGS zurck.
	 * @return Schluessel_C_TGS. 
	 */
	public byte[] getSchluessel_C_TGS()
	{
		if(Schluessel_C_TGS == null)
			throw new NullPointerException();
		return Schluessel_C_TGS; 
	}
	
	/**
	 * Gibt den Noncewert zurck. Besser checkNonce() verwenden.
	 * @return Nonce.
	 */
	public long getNonce(){ return Nonce; }

	/**
	 * @see kerberos.messages.Message#getType()
	 */
	public int getType()
	{
		return M_KDC_AS_REP;
	}
}
