/*
 * Created on Nov 3, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kerberos.messages;

import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.packets.PacketException;
import kerberos.packets.PacketReader;

/**
 * Dient zum Auslesen der Antwort des TGS an den Client.
 * 
 * @author stier_j
 */
public class M_KDC_TGS_REP extends Message {

	byte[] Schluessel_C_S = null, ticket = null;
	long Nonce = 0;
	
	M_KDC_TGS_REP(byte[] Daten, byte[] Schluessel) throws PacketException, PasswortFalschException
	{
		PacketReader pr = new PacketReader(Daten);
		byte[] daten = pr.readData(Schluessel);
			
		pr = new PacketReader(daten);
		Schluessel_C_S = pr.readDataNonCrypt();
		Nonce = pr.readNonceNonCrypt();
		ticket = pr.readDataNonCrypt();
	}
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.
	 * @throws NonceFalschException wenn die Noncewerte nicht bereinstimmen.
	 */	
	public void checkNonce(long mynonce) throws NonceFalschException
	{
		if(mynonce != Nonce) throw new NonceFalschException();
	}
	
	/**
	 * Gibt das Ticket fr den Applicationserver zurck.
	 * @return Ticket.
	 */
	public byte[] getTicket()
	{
		if(ticket == null)
		{
			throw new NullPointerException("M_KDC_TGS_REP.getTicket() ticket=null");
		} 
		return ticket; 
	}
	
	/**
	 * Gibt den Sitzungsschlssel zwischen dem Client und dem Applicationserver zurck.
	 * @return Schluessel_C_S.
	 */
	public byte[] getSchluessel()
	{
		if(Schluessel_C_S == null)
			throw new NullPointerException();
		return Schluessel_C_S; 
	}
	
	/**
	 * Gibt den ausgelesenen Noncewert zurck. Besser checkNonce() verwenden.
	 * @return Nonce.
	 */
	public long getNonce(){ return Nonce; }

	/**
	 * @see kerberos.messages.Message#getType()
	 */
	public int getType()
	{
		return M_KDC_TGS_REP;
	}
	
	public String toString()
	{
		return "Message vom KDC mit Ticket";
	}
}
