/*
 * Created on Nov 3, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kerberos.messages;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.packets.*;

/**
 * Dient zum Auslesen der Anfrage des Client an den TGS.
 * MessageTyp = 3;
 *  
 * @author stier_j
 */
public class M_KDC_TGS_REQ extends Message implements M_KDC_REQ, MessageWithTime
{
	private byte[] ticket = null, AuthClient = null, Schluessel_C_TGS = null;
	private String Client1 = null, TGS = null, Client2 = null, Server = null;
	private long Nonce = 0, startZeit = 0, endZeit = 0, zeit = 0; 
	
	M_KDC_TGS_REQ(byte[] Daten, byte[] Schluessel) throws PacketException,
		PasswortFalschException, ClientnameFalschException
	{
		PacketReader pr = new PacketReader(Daten);
		ticket = pr.readDataNonCrypt();
		PacketReader pr2 = new PacketReader(ticket);
		ticket = pr2.readData(Schluessel);
			
		PacketReader prt = new PacketReader(ticket);
		Client1 = prt.readStringNonCrypt();
		TGS = prt.readStringNonCrypt();
		startZeit = prt.readLongNonCrypt();
		endZeit = prt.readLongNonCrypt();
		Schluessel_C_TGS = prt.readDataNonCrypt();
			
		AuthClient = pr.readDataNonCrypt();
		pr2 = new PacketReader(AuthClient);
		AuthClient = pr2.readData(Schluessel_C_TGS);
			
		prt = new PacketReader(AuthClient);
		Client2 = prt.readStringNonCrypt();
		zeit = prt.readLongNonCrypt();
			
		Server = pr.readStringNonCrypt();
		Nonce = pr.readNonceNonCrypt();
			
		if(!Client1.equals(Client2))
			throw new ClientnameFalschException(Client1, Client2);
	}
	
	/**
	 * Gibt den ausgelesenen Clientnamen aus dem Ticket zurck.
	 * @return Client Name.
	 */
	public String getClientNameTicket()
	{
		if(Client1 == null)
			throw new NullPointerException();
		return Client1; 
	}
	
	/**
	 * Gibt den ausgelesenen Usernamen zurck.
	 * @return User Name.
	 */
	public String getUsername()
	{
		if(Client2 == null)
			throw new NullPointerException();
		return Client2; 
	}
	
	/**
	 * Gibt den ausgelesenen Servernamen aus dem Ticket zurck.
	 * Zum Vergleichen ob man auch gemeint ist. 
	 * @return TGS Namen.
	 */
	public String getServerName()
	{
		if(TGS == null)
			throw new NullPointerException();
		return TGS; 
	}
	
	/**
	 * Mit getEndZeit() vergleichen, ob Ticket noch im Zeitfenster ist.
	 * @return Gltigkeitsbeginn des Tickets.
	 */
	public long getStartzeit() { return startZeit; }
	
	/**
	 * Mit getStartZeit(), vergleichen ob Ticket noch im Zeitfenster ist.
	 * @return Ablaufzeit des Tickets.
	 */
	public long getEndzeit(){ return endZeit; }
	
	/**
	 * Aktueller Zeitpunkt um zu sehen, wann der Client geschickt hat.
	 * @return Zeit.
	 */
	public long getZeit() { return zeit; }
	
	/**
	 * Gibt den ausgelesenen Sitzungsschlssel zwischen Client und dem TGS zurck.
	 * @return Schluessel_C_TGS.
	 */
	public byte[] getSchluessel_C_TGS()
	{
		if(Schluessel_C_TGS == null)
			throw new NullPointerException();
		return Schluessel_C_TGS; 
	}
	
	/**
	 * Vergleicht den Noncewert mit dem Noncewert aus der Message.
	 * </br>
	 * <b>Hier nicht benutzt.</b>
	 */	
	public void checkNonce(long nonce){}
	
	/**
	 * Gibt das ausgelesene Ticket zurck. Wird eigentlich nicht bentigt.
	 * @return Ticket.
	 */
	public byte[] getTicket()
	{
		if(ticket == null)
			throw new NullPointerException();
		return ticket; 
	}
	
	/* *
	 * Gibt die ausgelesene Client Authentifikation zurck.
	 * Wird nicht bentigt.
	 * @return AuthClient.
	 * /
	private byte[] getAuthClient()
	{
		if(AuthClient == null)
			throw new NullPointerException();
		return AuthClient; 
	} //*/
	
	/**
	 * Gibt den ausgelesenen Zielservernamen zurck. Dient zur Feststellung an wen sich die
	 * eigentliche Anfrage des Clients richtet, und fr welchen Dienst das Ticket ausgestellt
	 * werden muss. 
	 * @return Zielservername.
	 */
	public String getZieldienstname()
	{
		if(Server == null)
			throw new NullPointerException();
		return Server; 
	}
	
	/**
	 * Gibt den ausgelesenen Noncewert zurck.
	 * @return Nonce.
	 */
	public long getNonce(){ return Nonce; }

	/**
	 * @see kerberos.messages.MessageWithTime#getType()
	 */
	public int getType()
	{
		return M_KDC_TGS_REQ;
	}
}
