/*
 * Created on Oct 20, 2003
 */
package kerberos.messages;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.net.Passwort;
import kerberos.packets.PacketBuilder;
import kerberos.packets.PacketException;

/**
 * Dient zum Erstellen von Nachrichten
 * @author Simon Seifer
 */
public class MessageBuilder implements IMessageBuilder
{
	public static final boolean DEBUG = false;
	
	private static final IMessageBuilder builder = new MessageBuilder();
	
	public static IMessageBuilder currentBuilder()
	{
		return builder;
	}
	
	/**
	 *  Die einzelnen Nachrichten-Methoden mssen noch um Exceptions erweitert werden
	 *  und evtl. noch ergnzt oder abgendert werden.
	 * 
	 * @param benutzername Name des Benutzers
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildKdcAuthLogin(String benutzername) throws PacketException
	{
		byte[] rt = null;
		long nonce = 0;
		
		PacketBuilder pb = new PacketBuilder(Message.M_KDC_AS_REQ);
		pb.addNotCrypt(benutzername);
		nonce = pb.addNonceNotCrypt();
		rt = pb.getPaket();

		return new Mess_mem_Nonce(rt,nonce);
	}
	
	/**
	 * 
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @param benutzername Name des Benutzers
	 * @param SchlsselTGS Schlssel des Ticket Granting Service 
	 * @param KDC Name des KDC
	 * @param startzeit Beginn der Gltigkeitsdauer des Tickets
	 * @param endzeit Ende der Gltigkeitsdauer des Tickets
	 * @param SchlsselClient Schlssel des Clients
	 * @param SchlsselClient_TGS Sitzungsschlssel zwischen Client und TGS
	 * @return byte[]
	 */
	public byte[] buildKdcAuthLoginOK(long nonce, String benutzername, byte[] SchlsselTGS, String KDC,
		long startzeit, long endzeit, byte[] SchlsselClient, byte[] SchlsselClient_TGS)
	throws PacketException
	{
		byte[] rt = null;
		byte[] ticket = null;

		PacketBuilder pb = new PacketBuilder();
		PacketBuilder pbt = new PacketBuilder(); // Ticket-Paket

		pbt.addNotCrypt(benutzername);
		pbt.addNotCrypt(KDC);
		pbt.addNotCrypt(startzeit);
		pbt.addNotCrypt(endzeit);
		pbt.addNotCrypt(SchlsselClient_TGS);
		rt = pbt.getPaket();

		pbt = new PacketBuilder();
		pbt.add(rt, SchlsselTGS); // Ticket verschlsseln
		ticket = pbt.getPaket(); // verschlsseltes Ticket

		pb.addNotCrypt(SchlsselClient_TGS);
		pb.addNotCrypt(nonce);
		pb.addNotCrypt(ticket);
		rt = pb.getPaket();

		pb = new PacketBuilder(Message.M_KDC_AS_REP);
		pb.add(rt, SchlsselClient);

		return pb.getPaket();
	}

	/**
	 * 
	 * @param TicketTGS Ticket fr den TGS
	 * @param benutzername Name des Benutzers
	 * @param SchlsselClient_TGS Sitzungsschlssel zwischen dem Client und dem TGS
	 * @param zieldienstname  ehemals servername gewesen
	 * @return Mess_mem_Nonce
	 */
	public Mess_mem_Nonce buildTGSLogin(byte[] TicketTGS, String benutzername, byte[] SchlsselClient_TGS,
		String zieldienstname) throws PacketException
	{
		byte[] rt = null;
		byte[] temp = null;
		long nonce = 0;

		PacketBuilder pb = new PacketBuilder(Message.M_KDC_TGS_REQ);
		PacketBuilder pbt = new PacketBuilder();

		pbt.addNotCrypt(benutzername);
		pbt.addNotCrypt(System.currentTimeMillis());
		rt = pbt.getPaket();

		pbt = new PacketBuilder();
		pbt.add(rt, SchlsselClient_TGS);
		temp = pbt.getPaket();

		pb.addNotCrypt(TicketTGS);
		pb.addNotCrypt(temp);
		pb.addNotCrypt(zieldienstname);
		nonce = pb.addNonceNotCrypt();

		rt = pb.getPaket();
		return new Mess_mem_Nonce(rt, nonce);
	}

	/**
	 * 
	 * @param SchluesselClient_Server
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @param benutzername Name des Benutzers
	 * @param servername
	 * @param startzeit
	 * @param endzeit
	 * @param SchluesselS
	 * @param SchluesselC_TGS
	 * @return byte[]
	 */
	public  byte[] buildTGSLoginOK(byte[] SchluesselClient_Server, long nonce, String benutzername, String servername,
		long startzeit, long endzeit, byte[] SchluesselS, byte[] SchluesselC_TGS) throws PacketException
	{
		if(DEBUG)
		{
			System.out.println("MessageBuilder.buildTGSLoginOK: schlssel{C,S}="+Passwort.toZahl(SchluesselClient_Server));
			System.out.println("MessageBuilder.buildTGSLoginOK: benutzername=" + benutzername);
			System.out.println("MessageBuilder.buildTGSLoginOK: servername=" + servername);
			System.out.println("MessageBuilder.buildTGSLoginOK: schlssel{S}=" + Passwort.toZahl(SchluesselS));
			System.out.println("MessageBuilder.buildTGSLoginOK: schlssel{C,TGS}=" + Passwort.toZahl(SchluesselC_TGS));
		}
		byte[] rt = null;

		PacketBuilder pb = new PacketBuilder(Message.M_KDC_TGS_REP);
		PacketBuilder pbt = new PacketBuilder();
		byte[] ticket = null;

		pbt.addNotCrypt(benutzername);
		pbt.addNotCrypt(servername);
		pbt.addNotCrypt(startzeit);
		pbt.addNotCrypt(endzeit);
		pbt.addNotCrypt(SchluesselClient_Server);
		rt = pbt.getPaket();

		pbt = new PacketBuilder();
		pbt.add(rt, SchluesselS);
		ticket = pbt.getPaket();

		pbt = new PacketBuilder();
		pbt.addNotCrypt(SchluesselClient_Server);
		pbt.addNotCrypt(nonce);
		pbt.addNotCrypt(ticket);
		rt = pbt.getPaket();

		pb.add(rt, SchluesselC_TGS);
		return pb.getPaket();
	}

	/**
	 * Vom Server zum Client<br>
	 * Message von Application zum Client
	 * mussten hier noch einen Schlssel in die bergabe einfgen 
	 * @param daten
	 * @param schluessel
	 * @return Mess_mem_Nonce
	 */

	public  Mess_mem_Nonce buildData(byte[] daten, byte[] schluessel) throws PacketException
	{
		byte[] rt = null;
		long nonce = 0;

		PacketBuilder pb = new PacketBuilder(Message.M_AP_REQ2);
		pb.add(daten, schluessel);
		nonce = pb.addNonceNotCrypt();
		rt = pb.getPaket();
		return new Mess_mem_Nonce(rt, nonce);
	}
	
	/**
	 * @param daten Zu verschlsselnde Daten
	 * @param schluessel Schlssel zum verschlsseln
	 * @param nonce Noncewert zur Erkennung von Wiedereinspielungsangriffen
	 * @return byte[]
	 */
	public  byte[] buildData(byte[] daten, byte[] schluessel, long nonce) throws PacketException
	{
		PacketBuilder pb = new PacketBuilder(Message.M_AP_REP);
		pb.add(daten, schluessel);
		pb.addNotCrypt(nonce);
		return pb.getPaket();
	}
	
	/**
	 * 
	 * @param Serverticket Ticket fr den Zielserver
	 * @param benutzername Name des Benutzers
	 * @param SchluesselClient_Server Sitzungsschlssel zwischen Client und Server
	 * @return Mess_mem_Nonce
	 * @throws PacketException
	 */
	public  Mess_mem_Nonce buildApLogin(byte[] Serverticket, String benutzername, byte[] SchluesselClient_Server) throws PacketException
	{
		return buildApLogin(Serverticket, benutzername, SchluesselClient_Server, new byte[0]);
	}
	
	/**
	 * Vom Client zum Server<br>
	 * @param serverticket Ticket an den Server
	 * @param benutzername Name des Benutzers
	 * @param SchluesselClient_Server Sitzungsschlssel zwischen Client und Zielserver
	 * @param daten ehemals Command
	 * @return Mess_mem_Nonce
	 */
	public  Mess_mem_Nonce buildApLogin(byte[] serverticket, String benutzername, byte[] SchluesselClient_Server,
		byte[] daten) throws PacketException
	{
		byte[] rt = null;
		long nonce = 0;

		PacketBuilder pb = new PacketBuilder(Message.M_AP_REQ);
		PacketBuilder pbt = new PacketBuilder();
		byte[] temp = null;

		pbt.addNotCrypt(benutzername);
		pbt.addNotCrypt(System.currentTimeMillis());
		rt = pbt.getPaket();

		pbt = new PacketBuilder();
		pbt.add(rt, SchluesselClient_Server);
		temp = pbt.getPaket();

		pb.addNotCrypt(serverticket);
		pb.addNotCrypt(temp);
		pb.add(daten, SchluesselClient_Server);
		nonce = pb.addNonceNotCrypt();
		rt = pb.getPaket();
		return new Mess_mem_Nonce(rt, nonce);
	}

	/**
	 * Message wenn Fehler aufgetreten ist.<br>
	 * Errorcodes siehe M_ERROR.java.
	 * 
	 * @param messageType Typ der Nachricht bei der der Fehler auftrat
	 * @param errorcode Art des Fehlers
	 * @return byte[]
	 */	
	public  byte[] buildError(int messageType, int errorcode) throws PacketException
	{
		PacketBuilder pb = new PacketBuilder(Message.M_ERROR);
		pb.addNotCrypt(messageType);
		pb.addNotCrypt(errorcode);
		return pb.getPaket();
	}

	/**
	 * @param message zu entschlsselnde Nachricht
	 * @param schluessel Schlssel zum entschlsseln
	 * @return Message
	 */	
	public  Message readMessage(byte[] message, byte[] schluessel)
		throws PasswortFalschException, PacketException, ClientnameFalschException, UngltigerMessageTypException
	{
		return Message.createMessage(message, schluessel);
	}
}
