/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.InetAddress;

import util.logger.*;

import kerberos.exceptions.ZahlNichtPositivException;

/**
 * Dieses Interface stellt ServerSockets, verschlssekt mit Kerberos, bereit
 * @author Stephan
 */
public interface KerberosServerSocket
{
	/**
	 * Nimmt eine Verbindung von einem Client an.
	 * @see java.net.ServerSocket#accept()
	 */
	public KerberosSocket accept() throws IOException;

	/**
	 * Ab dem Aufruf dieser Methode nimmt dieser KerberosServerSocket nimmt keine neuen Verbindungen mehr an.
	 * Auch schon angenommene Verbindungen und authentifizierte Verbindungen werden abgebrochen.
	 * @see #closeLigth()
	 * @see java.net.ServerSocket#close()
	 */
	public void close() throws IOException;

	/**
	 * Ab dem Aufruf dieser Methode nimmt dieser KerberosServerSocket nimmt keine neuen Verbindungen mehr an.
	 * Schon angenommene Verbindungen und authentifizierte knnen mit accept() abgerufen werden.
	 * @see #close() 
	 * @see java.net.ServerSocket#close()
	 */
	public void closeLigth() throws IOException;

	/**
	 * @return Gibt true zurck, wenn der ServerSocket ganz geschlossen ist,
	 * d.h. wenn keine Verbindungen mehr angenommen werden und auch keine schon 
	 * authentifizierten Verbindungen mehr auf ihr accept() warten. 
	 * @see java.net.ServerSocket#isClosed()
	 * @see #isClosedLight()
	 * @see #close()
	 */
	public boolean isClosed();

	/**
	 * @return Gibt true zurck, wenn der ServerSocket keine neuen Verbindungen mehr annimmt.
	 * Ob noch schon authorizierte Verbindungen auf ihr accept warten, wird nicht geprft.
	 * @see #isClosed() 
	 * @see #closeLigth()
	 */
	public boolean isClosedLight();

	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Nachrichten zurck, welche an die
	 * KerberosSockets der sich ab jetzt anmeldenden Clients bergeben wird.
	 */
	public int getMaxArrayLength();

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Nachrichten der erzeugten KerberosSockets.
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int i) throws ZahlNichtPositivException;

	/**
	 * Gibt die lokale IP-Adresse des Server-Sockets zurck.
	 */
	public InetAddress getInetAddress();
	
	/**
	 * Gibt den Port zurck, auf dem dieser Socket horcht.
	 * @see java.net.ServerSocket#getLocalPort()
	 */
	public int getLocalPort();

	/**
	 * Setzt den Logger, der ab sofort fr die neu erstellten KerberosSockets
	 * verwendet wird. Fr schon erzeugte Sockets msste der Logger dort explizit
	 * verndert werden. Deshalb sollte der Logger einmal am Anfang gesetzt werden.
	 * @param logger Wenn der Logger null ist, werden die Sachen nicht mehr geloggt.
	 */
	public void setLogger(Logger logger);

	/**
	 * Gibt den aktuell verwendeten Logger zurck
	 * @see #setLogger(Logger)
	 */
	public Logger getLogger();
}
