/*
 * Created on 24.11.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.Socket;

import util.logger.*;

import kerberos.exceptions.ErstEmpfangenException;
import kerberos.exceptions.ErstSendenException;
import kerberos.exceptions.NonceFalschException;
import kerberos.messages.M_AP;
import kerberos.messages.Mess_mem_Nonce;
import kerberos.messages.MessageBuilder;
import kerberos.packets.PacketException;

/**
 * KerberosSocket auf der Client-Seite
 * @see kerberos.net.KerberosSocket
 * @author Stephan Koops
 */
class KerberosSocketC extends KerberosSocketAbstr
{
	private static final boolean debug = false;
	
	/**
	 * Speichert den Nonce-Wert, mit dem die lezte Nachricht versendet wurde. 
	 */
	private Long lastNonce;
	
	private byte[] serverticket;
	
	/**
	 * @param socket
	 * @param schlssel
	 * @param username
	 * @throws IOException
	 */
	KerberosSocketC(Socket socket, byte[] schlssel_CS, String username, byte[] serverticket, int maxArrayLength, Logger logger) throws IOException
	{
		super(socket, schlssel_CS, username, maxArrayLength, logger);
		this.serverticket = serverticket;
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#buildCryptData(byte[])
	 */
	protected byte[] buildCryptData(byte[] clearData) throws PacketException, ErstEmpfangenException
	{
		boolean debug = false;
		String methode ="KerberosSocketC.buildCryptData";
		if(this.lastNonce != null) // Es ist noch ein Nonce-Wert gespeichert
			throw new ErstEmpfangenException();
		if (debug)
		{
			System.out.println(methode+": Verschlsseln mit this.schlssel{C,S} "+Passwort.toZahl(this.schlssel_CS));
			System.out.println(methode+": this.serverticket "+Passwort.toZahl(this.serverticket));
			System.out.println(methode+": Username="+this.username);
			System.out.println(methode+": Daten="+new String(clearData)+"<="+Passwort.toZahl(clearData));
		}
		Mess_mem_Nonce messNonce = MessageBuilder.currentBuilder().buildData(clearData, this.schlssel_CS);
		this.lastNonce = new Long(messNonce.nonce);
		if (debug)
			System.out.println("KerberosSocketC.buildCryptData(): Daten zu senden");
		return messNonce.message;
	}
	
	/**
	 * Testet auf der Client-Seite, ob der Nonce-Wert stimmt.
	 * Auf der Serverseite braucht nix passieren. 
	 * @param message
	 * @throws NonceFalschException Wird geworfen, wenn der Nonce-Wert falsch war.
	 * @throws NullPointerException Wird geworfen, wenn der letzte Nonce-Wert
	 * schon zum Testen einer Nachricht verwendet wurde, d.h. es muss erst wieder
	 * eine Nachricht zum Server gesendet werden, bevor dieser antworten kann.
	 * @see KerberosSocketAbstr#handleNonce(M_AP) 
	 */
	protected void handleNonce(M_AP message) throws NonceFalschException, ErstSendenException
	{
		if(this.lastNonce == null)
			throw new ErstSendenException();
		message.checkNonce(this.lastNonce.longValue());
		this.lastNonce = null;
		if(debug)
			System.out.println("KerberosSocketC.handleNonce(): Daten empfangen");
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#checkReadAllowed()
	 */
	protected void checkReadAllowed() throws ErstSendenException
	{
		if(this.lastNonce != null) // alles OK
			return;
		throw new ErstSendenException();
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#checkPrintAllowed()
	 */
	protected void checkPrintAllowed() throws ErstEmpfangenException
	{
		if(this.lastNonce == null) // alles OK
			return;
		throw new ErstEmpfangenException();
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#zustzlichePaketgreDurchPacketBuilder()
	 */
	int zustzlichePaketgreDurchPacketBuilder()
	{
		return 256; // Dummy-Wert
	}
}