/*
 * Created on 24.11.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.Socket;

import util.logger.*;

import kerberos.exceptions.ErstEmpfangenException;
import kerberos.exceptions.ErstSendenException;
import kerberos.messages.M_AP;
import kerberos.messages.MessageBuilder;
import kerberos.packets.PacketException;

/**
 * KerberosSocket auf der Server-Seite
 * @see kerberos.net.KerberosSocket
 * @author Stephan
 */
class KerberosSocketS extends KerberosSocketAbstr
{
	/**
	 * Speichert den Nonce-Wert, mit dem das letzte Paket gekommen ist.
	 * Wenn der letzte Nonce-Wert ausgelesen wurde, dann muss er auf null gesetzt werden
	 */
	private Long lastNonce;
	
	/**
	 * @param socket
	 * @param schlssel
	 * @param username
	 * @param maxArrayLength Maximale Lnge der zu empfangenden Arrays
	 * @throws IOException
	 */
	KerberosSocketS(Socket socket, byte[] schlssel_CS, String username, int maxArrayLength, Logger logger) throws IOException
	{
		super(socket, schlssel_CS, username, maxArrayLength, logger);
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#buildCryptData(byte[])
	 */
	protected byte[] buildCryptData(byte[] clearData) throws PacketException, ErstEmpfangenException
	{
		if(this.lastNonce == null)
			throw new ErstEmpfangenException();
		//System.out.println("KerberosSocketS.buildCryptData: Verschlsseln mit this.schlssel{C,S} "+Passwort.toZahl(this.schlssel_CS));
		byte[] cryptData = MessageBuilder.currentBuilder().buildData(clearData, this.schlssel_CS, this.lastNonce.longValue());
		this.lastNonce = null;
		return cryptData; 
	}

	/**
	 * Speichert den Nonce-Wert, damit er frs nchste Senden bereit steht.
	 * @throws ErstSendenException Wird geworfen, wenn schon ein nonce-Wert gespeichert ist.
	 * Dann muss erst eine Nachricht gesendet werden, damit wieder alles passt
	 * @see KerberosSocketAbstr#handleNonce(M_AP)
	 */
	protected void handleNonce(M_AP message) throws ErstSendenException
	{
		if (this.lastNonce != null)
			throw new ErstSendenException();
		this.lastNonce = new Long(message.getNonce());
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#checkReadAllowed()
	 */
	protected void checkReadAllowed() throws ErstSendenException
	{
		if (this.lastNonce == null) // alles OK
			return;
		throw new ErstSendenException();
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#checkPrintAllowed()
	 */
	protected void checkPrintAllowed() throws ErstEmpfangenException
	{
		if(this.lastNonce != null) // alles OK
			return;
		throw new ErstEmpfangenException();
	}

	/**
	 * @see kerberos.net.KerberosSocketAbstr#zustzlichePaketgreDurchPacketBuilder()
	 */
	int zustzlichePaketgreDurchPacketBuilder()
	{
		return 256; // Dummy-Wert
	}
}
