/*
 * Created on 07.10.2003
 */
package kerberos.net.test;

import java.io.IOException;

import util.logger.Logger;

import kerberos.exceptions.PasswortFalschException;
import kerberos.net.KerberosClientSocketFactory;
import kerberos.net.KerberosSocket;
import kerberos.net.Passwort;

/**
 * Diese Klasse Dient auf der Clientseite dazu, die Test-KerberosSockets zu erzeugen.
 * Bei diesen test-KerberosSockets stehen keine Benutzernamen zur Verfgung
 * @author Stephan Koops
 */
public class KerberosTestClientSocketFactory implements KerberosClientSocketFactory
{
	private int maxArrayLength = 256*256;
	
	/**
	 * Simuliert eine KerberosClientSocketFactory. Bentigt keinen KDC, da alles unverschlsselt ist.<br>
	 * @param username Benutzername im KDC
	 * @param passwort Dazugehriges Passwort im KDC
	 * @param kdcHostnameOrIP
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 * beim KDC Netzwerk-Probleme gab.
	 * @throws PasswortFalschException Wenn das angegebene Passwort falsch ist.
	 */
	public KerberosTestClientSocketFactory(String username, Passwort passwort, String kdcHostnameOrIP)
		throws IOException, PasswortFalschException
	{
		KerberosTestSocket.tuNix(username);
		KerberosTestSocket.tuNix(passwort);
		KerberosTestSocket.tuNix(kdcHostnameOrIP);
	}

	/**
	 * Baut eine Verbindung zum Ziel-Server auf
	 * @param hostname Hostname des Ziel-Servers
	 * @param port Ziel-Port
	 * @param zieldienst Name des Ziel-Dienstes im KDC
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 * beim KDC Netzwerk-Probleme gab.
	 */
	public KerberosSocket connect(String hostname, int port, String zieldienst) throws IOException
	{
		return new KerberosTestSocket(hostname, port, zieldienst, this.maxArrayLength);
	}

	/**
	 * Zerstrt die Tickets, so dass diese garantiert aus dem RAM gelscht werden.
	 */
	public void destroy()
	{
	}

	/**
	 * @see java.lang.Object#finalize()
	 */
	public void finalize() throws Throwable
	{
		this.destroy();
		super.finalize();
	}
	
	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Arrays der erzeugten KerberosSockets zurck.
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Nachrichten
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int i)
	{
		this.maxArrayLength = i;
	}

	public void setLogger(Logger logger) 
	{
	}

	public Logger getLogger() 
	{
		return null;
	}
}
