/*
 * Created on 07.10.2003
 */
package kerberos.net.test;

import java.io.*;
import java.net.*;

import util.logger.*;

import kerberos.exceptions.*;
import kerberos.net.*;

/**
 * Unverschlsselter Server-Socket, der aber das gleiche Interface hat wie die KerberosSockets.<br>
 * Kennt keinen Logger
 * @author Stephan
 */
public class KerberosTestServerSocket implements KerberosServerSocket
{
	ServerSocket serverSocket;
	
	private int maxArrayLength = 256*256; 
	
	/**
	 * Erzeugt einen unverschlsselten Server-Socket, der aber das gleiche Interface hat wie die KerberosSockets
	 */
	protected KerberosTestServerSocket(ServerSocket serverSocket, Passwort passwort) throws PasswortFalschException
	{
		this.serverSocket = serverSocket;
		KerberosTestSocket.tuNix(passwort);
	}

	public KerberosTestServerSocket(int port, Passwort passwort)
		throws IOException, PasswortFalschException
	{
		this(new ServerSocket(port), passwort);
	}

	public KerberosTestServerSocket(int port, int backlog, Passwort passwort)
		throws IOException, PasswortFalschException
	{
		this(new ServerSocket(port, backlog), passwort);
	}

	public KerberosTestServerSocket(int port, int backlog, InetAddress bindAddr, Passwort passwort)
		throws IOException, PasswortFalschException
	{
		this(new ServerSocket(port,backlog,bindAddr),passwort);
	}
	
	/**
	 * Gibt einen KerberosTestSocket zurck
	 * @see java.net.ServerSocket#accept()
	 */
	public KerberosSocket accept() throws IOException
	{
		return new KerberosTestSocket(this.serverSocket.accept(), maxArrayLength);
	}

	/**
	 * Schliet die Verbindung
	 * @see java.net.ServerSocket#close()
	 */
	public void close() throws IOException
	{
		this.closeLigth();
	}

	/**
	 * @see kerberos.net.KerberosServerSocket#closeLigth()
	 */
	public void closeLigth() throws IOException
	{
		this.serverSocket.close();		
	}

	// Goodies

	/**
	 * Returns the local address of this server socket.
	 *
	 * @return  the address to which this socket is bound,
	 *          or <code>null</code> if the socket is unbound.
	 */
	public InetAddress getInetAddress()
	{
		return this.serverSocket.getInetAddress();
	}

	public int getLocalPort()
	{
		return this.serverSocket.getLocalPort();
	}

	/**
	 * Returns the closed state of the ServerSocket.
	 * @return true if the socket has been closed
	 */
	public boolean isClosed()
	{
		return this.serverSocket.isClosed();
	}

	/**
	 * @see java.net.ServerSocket#setSoTimeout(int)
	 */
	public void setSoTimeout(int timeout) throws SocketException
	{
		this.setSoTimeout(timeout);
	}

	/** 
	 * @see java.net.ServerSocket#getSoTimeout()
	 */
	public int getSoTimeout() throws IOException
	{
		return this.serverSocket.getSoTimeout();
	}

	/**
	 * Returns the implementation address and implementation port of 
	 * this socket as a <code>String</code>.
	 *
	 * @return  a string representation of this socket.
	 */
	public String toString()
	{
		return this.serverSocket.toString();
	}

	/**
	 * @see java.net.ServerSocket#setReceiveBufferSize(int)
	 */
	public void setReceiveBufferSize(int size)
		throws SocketException
	{
		this.serverSocket.setReceiveBufferSize(size);
	}

	/**
	 * @see java.net.ServerSocket#getReceiveBufferSize()
	 */
	public int getReceiveBufferSize() throws SocketException
	{
		return this.serverSocket.getReceiveBufferSize();
	}

	/**
	 * @see kerberos.net.KerberosServerSocket#isClosedLight()
	 */
	public boolean isClosedLight()
	{
		return this.isClosed();
	}

	/**
	 * @see kerberos.net.KerberosServerSocket#getMaxArrayLength()
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * @see kerberos.net.KerberosServerSocket#setMaxArrayLength(int)
	 */
	public void setMaxArrayLength(int i)
	{
		this.maxArrayLength = i;
	}

	public void setLogger(Logger logger)
	{
	}

	public Logger getLogger() 
	{
		return null;
	}
}
