/*
 * Created on 07.10.2003
 */
package kerberos.net.test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;

import kerberos.exceptions.SendeArrayZuLangException;
import kerberos.net.KerberosSocket;
import sun.net.ConnectionResetException;
import util.NetUtil;

/**
 * Diese Klasse dient fr eine konkrete Verbindung zu einem Client. Diese ist dann schon aufgebaut
 * und steht erfolgreich.
 * @author Stephan Koops
 */
public class KerberosTestSocket implements KerberosSocket
{
	protected Socket socket;
	protected PrintWriter writer;
	protected BufferedReader reader;
	
	private int maxArrayLength; 
	
	/**
	 * Baut eine Verbindung zum Zieldienst mit dem angegebenen Port auf dem Ziel-Server auf
	 * @param hostname Hostname des Ziel-Servers
	 * @param port Ziel-Port
	 * @param zieldienst Name des Ziel-Dienstes im KDC
	 * @throws IOException
	 */
	KerberosTestSocket(String hostname, int port, String zieldienst, int maxArrayLength) throws IOException
	{
		this(new Socket(hostname,port), zieldienst, maxArrayLength);
	}

	/**
	 * Benutzt fr die Kerberos-Verbindung einen vorhandenen Socket.
	 * Dieser darf danach nicht mehr anderweitig benutzt werden !!
	 * @param socket
	 * @param zieldienst
	 * @throws IOException
	 */
	KerberosTestSocket(Socket socket, String zieldienst, int maxArrayLength) throws IOException
	{
		this(socket, maxArrayLength);
		tuNix(zieldienst);
	}

	/**
	 * diesen Konstruktor benutzt der KerberosServerSocket, wenn er einen neuen Client akzeptiert
	 * @param socket
	 * @throws IOException
	 */
	KerberosTestSocket(Socket socket, int maxArrayLength) throws IOException
	{
		this.socket = socket;
		this.writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())));
		this.reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		this.maxArrayLength = maxArrayLength;
	}

	/**
	 * Schreibt ein Byte-Array.
	 * @param zeichen
	 * @throws IOException
	 */
	public void print(String zeichen) throws IOException
	{
		if (zeichen == null)
			this.print(new byte[0]);
		else
			this.print(zeichen.getBytes());
	}

	/**
	 * Schreibt ein Byte-Array.
	 * @param zeichen
	 * @throws IOException
	 */
	public void print(byte[] zeichen) throws IOException
	{
		if(zeichen == null)
			zeichen = new byte[0];
		NetUtil.writeArray(this.socket, zeichen);
	}

	public byte[] printRead(byte[] zeichen) throws ConnectionResetException, SendeArrayZuLangException, IOException
	{
		this.print(zeichen);
		return this.read();
	}

	public String printRead(String zeichen) throws ConnectionResetException, SendeArrayZuLangException, IOException
	{
		this.print(zeichen);
		return this.readString();
	}

	public byte[] read() throws ConnectionResetException, SendeArrayZuLangException, IOException
	{
		return NetUtil.readByteArray(socket, maxArrayLength);
	}

	public String readString() throws ConnectionResetException, SendeArrayZuLangException, IOException
	{
		return new String(NetUtil.readByteArray(socket, maxArrayLength));
	}

	/**
	 * @see kerberos.net.KerberosSocket#close()
	 */
	public void close() throws IOException
	{
		writer.close();
		reader.close();
		socket.close();
	}

	/**
	 * @see kerberos.net.KerberosSocket#isClosed()
	 */
	public boolean isClosed()
	{
		return socket.isClosed();
	}

	/**
	 * @see kerberos.net.KerberosSocket#getUsername()
	 */
	public String getUsername()
	{
		return "testuser";
	}

	/**
	 * @see kerberos.net.KerberosSocket#getInetAddress()
	 */
	public InetAddress getInetAddress()
	{
		return this.socket.getInetAddress();
	}

	/**
	 * @see kerberos.net.KerberosSocket#getPort()
	 */
	public int getPort()
	{
		return this.socket.getPort();
	}

	/**
	 * @see kerberos.net.KerberosSocket#getMaxArrayLength()
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * @see kerberos.net.KerberosSocket#setMaxArrayLength(int)
	 */
	public void setMaxArrayLength(int i)
	{
		this.maxArrayLength = i;
	}

	public static void tuNix(Object o)
	{
		if(o != null)
			o.getClass();
	}
}