/*
 * Created on 10.01.2004
 */
package kerberos.server;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Singelton-Pattern<br>
 * ber die Methoden <code>log</code> wird falls notwendig eine Instance erzeugt.
 * Diese schaut in der KDCProperties, falls vorhanden, nach in welche Datei er die 
 * Informationen loggen soll. Standardmig wird System.out genutzt, wenn nichts weiter
 * angegeben ist.
 * @author Sven Elvers
 */
public class KDCLogger {

	//Schlssel fr die KDC.properties
	private static final String FILEPATH_KEY = "<KDCLogger-File>";

	//Klassenkonstanten
	private static final String CRLF = "\r\n";

	//Klassenvariablen
	private static KDCLogger Instance = null;

	//Instancevariablen
	private BufferedWriter out = null;
	private File file = null;
	
	/**
	 * ist wegen Singeltonpatern als private deklariert<br>
	 * erstellt einen Stream zu einer Datei, wenn in den Properties angegeben,<br>
	 * ansonsten wird der Standard-OutputStream genutzt.
	 */
	private KDCLogger() {
		KDCProperties p = KDCProperties.getInstace();
		OutputStreamWriter w;
		try {
			file = new File(p.getString(FILEPATH_KEY));
			if(!file.exists())
				file.createNewFile();
//			throw new Exception("Die Datei KDC.properties fehlt");
			w = new FileWriter(file,true);
			
		} catch (Exception e) {
			w = new OutputStreamWriter(System.out);
			file = null;
		}
		out = new BufferedWriter(w);
	}
	
	/**
	 * schreibt den gegebenen String in den OutputStream
	 * mit Zeitangabe vorweg (yyyy.MM.dd HH:mm:ss).<br>
	 * Erstellt eine neue Instance wenn keine vorhanden ist.
	 * @param str
	 */
	public static void log(String str){
		if (Instance == null)
			Instance = new KDCLogger();
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss '>>' ");
		String d = sdf.format(new Date());
		
		try {
			Instance.out.write(d + str + CRLF);
			Instance.out.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * schliet den OutputStream und 
	 * setzt die Klassenvariable Instance auf null. 
	 */
	public static void close() {
		try {
			Instance.out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		Instance = null;
	}

	/**
	 * schreibt in die erste Zeile das Datum
	 * und schreibt darunter den Stacktrace der Exception.
	 * @param e
	 * @see log(String str)
	 */
	public static void log(Exception e) {
		log("");
		try {
			if(Instance.file != null){
				FileOutputStream out = new FileOutputStream(Instance.file);
				PrintStream ps = new PrintStream(out);
				e.printStackTrace(ps);
				out.flush();
				out.close();
				ps.close();
				return;
			} 
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		e.printStackTrace();
	}

}
