/*
 * Created on 03.11.2003
 *
 */
package kerberos.server;

import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kerberos.server.keyaccess.AbstractKeyAccessor;
import kerberos.server.keyaccess.IKeyAccessor;

/**
 * Diese Klasse horcht den KDCPort ab, auf eingehende Verbindungen,<br>
 * die dann an einen KDCConnection-Thread bergeben wird.
 * @author Sven Elvers
 */
public class KDCServer extends Thread {

	//Klassenkonstanten
	public static final int PORT = 80;

	//Instancevariablen
	protected KDCServerSocket serverSocket;
	protected List connections = new ArrayList();
	
	/**
	 * Wird durch die Methode <code>start()</code> aufgerufen.
	 * @see java.lang.Thread#run()
	 */
	public void run() {
		super.run();
		try {
			IKeyAccessor keys = AbstractKeyAccessor.getKeyAccessor();
			serverSocket = new KDCServerSocket(
								PORT,
								keys.getKey("TGS")
								);
			
			handleRequests();
		} catch (IOException e) {
			KDCLogger.log(e);
		}
		System.out.println("KDCServer is down");
	}
	
	/**
	 * Es werden eingehende Verbindungen an eine KDCConnection zur Bearbeitung bergeben,
	 * bis der Thread beendet werden soll.
	 */
	private void handleRequests()
	{
		boolean noexception = true;
		IConnectionListener listener = new ConnectionListener();

		while (!this.isInterrupted() && noexception)
		{
			try {
				KDCLogger.log("KDCServer: Warte auf Verbindung");
				KDCSocket socket = serverSocket.accept();
				KDCLogger.log("KDCServer: Verbindung angenommen");
				KDCConnection con = new KDCConnection(socket);
				con.addEventListener(listener);
				connections.add(con);
				con.start();
			} catch (IOException e) {
				noexception = false;
				if (!this.isInterrupted())
				{
					KDCLogger.log(e);
				}
			}
		}
	}
	
	/**
	 * Ein ConnectionListener um mitzubekommen welche
	 * Threads/Verbindungen beendet sind.
	 * @author Samuel Mumm
	 */
	private class ConnectionListener implements IConnectionListener{

		/**
		 * @see kerberos.server.IConnectionListener#lostConnection(java.lang.Object)
		 */
		public void lostConnection(Object obj) {
			connections.remove(obj);
		}
		
	}

	/**
	 * Setzt eine interrupted Flag fr den Thread, dass dieser sich beenden soll.<br>
	 * Zustzlich beendet er den Serversocket, lscht die Referenz auf die Datenbank mit den Schlsseln<br>
	 * und sagt den noch laufenden Connections, dass sie ebenfalls sich beenden sollen.
	 * @see java.lang.Thread#interrupt()
	 */
	public void interrupt() {
		System.out.println("Authentification-Service is interrupted");
		super.interrupt();
		
		try {
			if (serverSocket != null) serverSocket.close();
		} catch (IOException e) {}
		
		Iterator iter = connections.iterator();
		while (iter.hasNext())
			((Thread) iter.next()).interrupt();
	}

	/**
	 * Gibt an auf welchem Port der KDC luft und wie viele Verbindungen offen sind.
	 * @param stream
	 * the outputstream
	 */
	public void printInfoOn(PrintStream stream) {
		stream.println("Kerberos uses port: " + PORT);
		stream.println("Kerberos has " + connections.size() + " running connections");
		
	}

}
