/*
 * Created on 20.10.2003
 */
package kerberos.server;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Dieser ServerSocket wird im KDC verwendet.
 * @author Stephan Koops
 */
public class KDCServerSocket
{
	public static final boolean debug = false;
	
	private ServerSocket serverSocket;
	private byte[] schlssel;

	/**
	 * Erzeugt einen KDC-Server-Socket, auf dem er neue Verbindungen erwartet.
	 * Dieser Konstruktor wird vom TGS benutzt
	 * @see java.net.ServerSocket#ServerSocket(int)
	 * @param port Port, auf dem die Verbindungen erwartet werden
	 * @throws IOException
	 */
	public KDCServerSocket(int port, byte[] schlssel) throws IOException
	{
		this.serverSocket = new ServerSocket(port);
		this.schlssel = schlssel;
	}

	/**
	 * @see java.net.ServerSocket#accept()
	 */
	public KDCSocket accept() throws IOException
	{
		Socket socket;
		for(;;)
		{
			socket = this.serverSocket.accept();
			if(debug)
				KDCLogger.log("KDCServerSocket.accept(): Verbindung angenommen ("+socket+")");
			return new KDCSocket(socket, this.schlssel);
		}
	}

	/**
	 * @see java.net.ServerSocket#close()
	 */
	public void close() throws IOException
	{
		this.serverSocket.close();
	}

	/**
	 * @see java.net.ServerSocket#getInetAddress()
	 */
	public InetAddress getInetAddress()
	{
		return this.serverSocket.getInetAddress();
	}

	/**
	 * @see java.net.ServerSocket#isClosed()
	 */
	public boolean isClosed()
	{
		return this.serverSocket.isClosed();
	}
}
