/*
 * Created on 01.11.2003
 *
 */
package kerberos.server.keyaccess;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import kerberos.server.KDCLogger;
import kerberos.server.KDCProperties;

/**
 * KeyAccessor zu einer Datei mit den Prinzipals und Schlsseln
 * @see java.util.Properties
 * @author Sven Elvers, Christian Morgenstern
 *
 */
public class KeyFileAccessor extends AbstractKeyAccessor {

	//Schlssel fr die KDC.properties
	public static final String FILEPATH_KEY = "<KeyFileAccessor-File>";

	//Klassenvariablen
	private static KeyFileAccessor Instance = null;

	//Instancevariablen
	private Properties keydatabase;
	protected String filename;

	/**
	 * @return Singelton der Klasse oder
	 * null wenn keine erstellt werden konnte
	 */
	public static KeyFileAccessor getInstace(){
		if (Instance == null) {
			try {
				Instance = new KeyFileAccessor();
			} catch (Exception e) {
				System.out.println(e.getMessage());
			}
		}
		
		return Instance; 
	}

	/**
	 * @throws Exception
	 * wenn die Datei nicht korrekt eingelesen werden konnte
	 */
	private KeyFileAccessor() throws Exception {
		KDCProperties p = KDCProperties.getInstace();
		filename = p.getString(FILEPATH_KEY);
		try {
			keydatabase = new Properties();
			FileInputStream stream = new FileInputStream(filename);
			keydatabase.load(stream);
			stream.close();
		} catch (FileNotFoundException e) {
			KDCLogger.log(e);
			throw new Exception("Cannot find " + filename);
		} catch (IOException e) {
			KDCLogger.log(e);
			throw new Exception("Cannot access " + filename);
		}
	}

	/**
	 * @see kerberos.server.keyaccess.IKeyAccessor#getKey(java.lang.String)
	 */
	public byte[] getKey(String user) {
		if(!keydatabase.containsKey(user)) return null;
		return stringToByteArray((String)keydatabase.get(user));
	}

	/**
	 * @param string
	 * @return
	 */
	private byte[] stringToByteArray(String string) {
		StringTokenizer st = new StringTokenizer(string);
		List list = new ArrayList();
		while(st.hasMoreTokens()){
			list.add(st.nextToken());
		}
		
		byte[] result = new byte[list.size()];
		
		for (int i = 0;i < result.length; i++){
			result[i] = Byte.parseByte((String)list.get(i),10);
		}
		
		return result;
	}

	/**
	 * @see kerberos.server.keyaccess.IKeyAccessor#changeKey(java.lang.String, byte[])
	 * @return True wenn gendert, false falls der User nicht existiert
	 */
	public boolean changeKey(String user, byte[] key) throws Exception {
		if(!keydatabase.containsKey(user)) return false;
		keydatabase.put(user, key);
		saveIt();
		return true;
	}

	/**
	 * @see kerberos.server.keyaccess.IKeyAccessor#addToken(java.lang.String, byte[])
	 * @return True wenn hinzugefgt, false falls der User schon existiert
	 */
	public boolean addToken(String user, byte[] key) throws Exception {
		if(keydatabase.containsKey(user)) return false;
		keydatabase.put(user, byteArrayToString(key));
		saveIt();
		return true;
	}

	/**
	 * @param key
	 * @return
	 */
	private String byteArrayToString(byte[] key) {
		String result = "";
		for (int i = 0; i < key.length; i++){
			result += Byte.toString(key[i]) + " ";
		}
		return result;
	}

	/**
	 * @see kerberos.server.keyaccess.IKeyAccessor#removeToken(java.lang.String)
	 * @return True wenn gelscht, false falls der User nicht existiert
	 */
	public boolean removeToken(String user) throws Exception
	{
		return keydatabase.remove(user) != null;
	}

	/**
	 * speichern der Schlsselbank
	 * @throws Exception
	 */
	private void saveIt() throws Exception
	{
		FileOutputStream stream = new FileOutputStream(filename);
		keydatabase.store(stream,"");
		stream.close();
	}	

}
