/*
 * Created on 02.12.2003
 */
package kerberos.tests.messages;

import junit.framework.Test;
import junit.framework.TestSuite;
import kerberos.exceptions.KerberosException;
import kerberos.messages.M_AP_REP;
import kerberos.messages.M_AP_REQ;
import kerberos.messages.M_AP_REQ2;
import kerberos.messages.M_ERROR;
import kerberos.messages.M_KDC_AS_REP;
import kerberos.messages.M_KDC_AS_REQ;
import kerberos.messages.M_KDC_TGS_REP;
import kerberos.messages.M_KDC_TGS_REQ;
import kerberos.messages.Mess_mem_Nonce;
import kerberos.messages.Message;
import kerberos.messages.MessageBuilder;
import kerberos.packets.PacketException;
import kerberos.packets.PacketReader;
import kerberos.tests.TestCase;

/**
 * Diese Klasse erlutert die Funktionsweise von JUnit anhand des Beispieles der Methode
 * Message.buildTgsLogin(...)
 * @author Stephan
 */
public class MessageBuilderTest extends TestCase // Jede Test-Klasse erbt von der Classe junit.framework.TestCase
{
	
	// Hier folgen Variabeln, die in der Methode setUp gefllt werden. (siehe dort)
	//public static final byte[] schlssel = "lkjgkhgjkg".getBytes();
	//public static final byte[] ticket = "kjhlgh dfkhskghkgjhdlk".getBytes();

	public byte[] TGSTicket = null;
	public byte[] Key_TGS = null;
	
	
	/**
	 * Constructor for MessageBuilderTest.
	 * @param arg0
	 */
	public MessageBuilderTest(String arg0) // Dieser Konstruktor muss vorhanden sein
	{
		super(arg0);
	}

	/**
	 * Jede Methode die mit test...() anfngt, wird einmal ausgefhrt.
	 * Vorher wird jedesmal die Methode setUp() ausgefhrt, so dass fr jeden Test
	 * (Methoden test....() die gleichen Vorraussetzungen gelten
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
	}
	
	public static Test suite()
	{
		return new TestSuite(MessageBuilderTest.class);
	}
	
	public void testBuildKdcAuthLogin() throws Exception
	{
		Mess_mem_Nonce mmn = MessageBuilder.currentBuilder().buildKdcAuthLogin("Alfred");
		long nonce1=mmn.nonce;
	
		Message message = MessageBuilder.currentBuilder().readMessage(mmn.message, null);
		
		assertTrue(message instanceof M_KDC_AS_REQ);
		M_KDC_AS_REQ KdcAuthLogin = (M_KDC_AS_REQ) message;
	
		assertEquals(KdcAuthLogin.getUsername(),"Alfred"); 
		assertEquals(KdcAuthLogin.getNonce(),nonce1);
	}
	
	public void testBuildKdcAuthLoginOK() throws Exception
	{
		byte[] schlssel_C_TGS = "k_Cl_TGS".getBytes();
		byte[] schlssel_C = "k_Client".getBytes();
		byte[] messageArray = MessageBuilder.currentBuilder().buildKdcAuthLoginOK(100001,"Alfred","key_TGS1".getBytes(),"KDC",1200,1600,schlssel_C, schlssel_C_TGS);
		Message message1 = MessageBuilder.currentBuilder().readMessage(messageArray, schlssel_C);
	
		assertTrue(message1 instanceof M_KDC_AS_REP);
		M_KDC_AS_REP kdcAuthLoginOK = (M_KDC_AS_REP) message1;
	
		TGSTicket = kdcAuthLoginOK.getTicket();
		// test
		PacketReader readp = new PacketReader(TGSTicket);
		byte[] arr2 = readp.readData("key_TGS1".getBytes());
		readp = new PacketReader(arr2);
		byte[] ba = "Alfred".getBytes();
		byte[] ba2= readp.readDataNonCrypt();
		assertEquals(ba,ba2);
	
		assertEquals(100001,kdcAuthLoginOK.getNonce());
		assertEquals(kdcAuthLoginOK.getSchluessel_C_TGS(), schlssel_C_TGS);
	}

	// Dieser Test wird zur Zeit nicht ausgefhrt, da das TGSTicket null ist
	public void _testBuildTGSLogin() throws Exception
	{
		// bentigte Variablen setzen
		Mess_mem_Nonce mmn = MessageBuilder.currentBuilder().buildTGSLogin(TGSTicket, "Alfred", "k_Cl_TGS".getBytes(), "zieldienstname");
		//long nonce2 = mmn.nonce;
		System.out.println("MMN.Nonce: "+mmn.nonce);
		Message message = MessageBuilder.currentBuilder().readMessage(mmn.message, "key_TGS1".getBytes());
		
		// Tests: die wesentlichen Methoden in den TestCases sind die Methoden
		// assert...(..). Diesen kann man ein paar Parameter bergeben, und wenn das
		// Ergebnis nicht passt (z.B. zwei Werte bei assertEquals nicht gleich sind),
		// dann wird angezeigt, wo der Fehler aufgetreten ist.
		// In dem View JUnit wird der Balken grn, wenn alle Tests ohne Fehler durchgelaufen sind, sonst wird der Balken rot.
		// weiter unten (Im Failure Trace) sieht man fr jeden Test, der nicht erfolgreich war,
		// was fr ein Fehler aufgetreten ist.
		// Wenn man auf dort auf die Zeile direkt unter der
		// Exception klickt, dann landet man dort, wo der Fehler aufgetreten ist.
		// Gestartet wird der Test fast wie eine normale Klasse:
		// Test-Klasse ffnen, "Run", "Run as", "JUnit Test". Dann msste links auch das
		// entsprechende Fenster fr die Tests aufgehen.
		assertTrue(message instanceof M_KDC_TGS_REQ); // Hier wird true erwartet (assert = erwarten)
		M_KDC_TGS_REQ tgsLogin = (M_KDC_TGS_REQ) message;
		assertEquals(tgsLogin.getZieldienstname(),"zieldienstname");
		assertEquals(tgsLogin.getUsername(),"Alfred");
		assertEquals(tgsLogin.getClientNameTicket(),"Alfred");
		assertEquals(tgsLogin.getNonce(),mmn.nonce);
	}

	public void testBuildTGSLoginOK() throws Exception
	{
		//buildTGSLoginOK
		byte[] Array2 = MessageBuilder.currentBuilder().buildTGSLoginOK("key_Cl_S".getBytes(),200,"Alfred","Server",
			100,200,"key_serv".getBytes(),"k_Cl_TGS".getBytes());
		Message message2 = MessageBuilder.currentBuilder().readMessage(Array2,"k_Cl_TGS".getBytes());
		assertTrue(message2 instanceof M_KDC_TGS_REP);
		M_KDC_TGS_REP TGSLoginOK = (M_KDC_TGS_REP) message2;
		assertEquals("key_Cl_S".getBytes(),TGSLoginOK.getSchluessel());
		byte[] sticket = TGSLoginOK.getTicket();


		//buildData
		Mess_mem_Nonce mmn2 = MessageBuilder.currentBuilder().buildApLogin(sticket, "Alfred", "key_Cl_S".getBytes(),
			"DatenTest".getBytes());
		long nonce3 = mmn2.nonce;
		Message message3 = MessageBuilder.currentBuilder().readMessage(mmn2.message, "key_serv".getBytes());
		assertTrue(message3 instanceof M_AP_REQ);
		M_AP_REQ Dat1 = (M_AP_REQ) message3;
		assertEquals("DatenTest".getBytes(),Dat1.getDaten());
		assertEquals(100,Dat1.getStartzeit());
		assertEquals(Dat1.getNonce(),mmn2.nonce);
		assertEquals(Dat1.getUsername(),"Alfred");


		//buildData2
		byte[] Array3 = MessageBuilder.currentBuilder().buildData("hallooo".getBytes(),"key_Cl_S".getBytes(),nonce3);	
		Message message4 = MessageBuilder.currentBuilder().readMessage(Array3,"key_Cl_S".getBytes());
		assertTrue(message4 instanceof M_AP_REP);
		M_AP_REP AP_REP = (M_AP_REP) message4;
		assertEquals(AP_REP.getDaten(),"hallooo".getBytes());
		assertEquals(AP_REP.getNonce(),nonce3);
	}
	
	public void testBuildData3() throws KerberosException, PacketException
	{	
		//build Data3
		Mess_mem_Nonce mmn3 = MessageBuilder.currentBuilder().buildData("halloo1".getBytes(),"key_Cl_S".getBytes());	
		
		Message message5 = MessageBuilder.currentBuilder().readMessage(mmn3.message,"key_Cl_S".getBytes());

		assertTrue(message5 instanceof M_AP_REQ2);
		M_AP_REQ2 AP_REQ2 = (M_AP_REQ2) message5;

		assertEquals(AP_REQ2.getDaten(),"halloo1".getBytes());
		assertEquals(AP_REQ2.getNonce(),mmn3.nonce);
	}

	/*public void testBuildErrorInt() throws Exception
	{
		byte[] Array1 = MessageBuilder.currentBuilder().buildError(Message.M_KDC_TGS_REQ, 12345);
		Message message = MessageBuilder.currentBuilder().readMessage(Array1,null);
		
		assertTrue(message instanceof M_ERROR);
		M_ERROR tgsLogin = (M_ERROR) message;
		assertEquals(Message.M_KDC_TGS_REQ, tgsLogin.getMessageType());
		assertEquals(12345, tgsLogin.getErrorcode());
	} //*/

	public void testBuildErrorByte() throws Exception
	{
		byte[] Array1 = MessageBuilder.currentBuilder().buildError(Message.M_KDC_TGS_REQ, 45);
		Message message = MessageBuilder.currentBuilder().readMessage(Array1,null);
		
		assertTrue(message instanceof M_ERROR);
		M_ERROR tgsLogin = (M_ERROR) message;
		assertEquals(Message.M_KDC_TGS_REQ, tgsLogin.getMessageType());
		assertEquals(45, tgsLogin.getErrorcode());
	}
}
