/*
 * Created on 20.10.2003
 */
package kerberos.tests.packets;

import java.util.Random;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import kerberos.exceptions.PasswortFalschException;
import kerberos.messages.Message;
import kerberos.packets.IPacketBuilder;
import kerberos.packets.IPacketReader;
import kerberos.packets.PacketBuilder;
import kerberos.packets.PacketReader;

/**
 * @author Christian Morgenstern
 */
public class PacketTests extends TestCase
{
	public boolean output = false;

	public PacketTests(String name)
	{
		super(name);
	}

	/**
	 * Um die Klasse als Anwendung zu starten
	 */
	protected PacketTests()
	{
		super();
		this.output = true;
	}

	public static Test suite()
	{
		return new TestSuite(PacketTests.class);
	}

	public void setUp()
	{
	}

	public static void main(String[] args) throws Exception
	{
		new PacketTests().test1();
	}
	
	public void test1() throws Exception
	{
		try{
		IPacketBuilder pb = new PacketBuilder( Message.M_FIRST );
		byte[] inByte = new byte[3];
		inByte[0] = (byte)12; inByte[1] = (byte)50; inByte[0] = (byte)64;
		long inLong = 545094534543L;
		long inNonce1, inNonce2;
		String inString = "Dies ist ein Test.";
		byte[] inKey = new byte[8];
		Random rnd = new Random();
		for( int i = 0 ; i < 8 ; i++ )
			inKey[i] = (byte)rnd.nextInt(255);
			
		pb.add( inByte, inKey );
		pb.add( inLong, inKey );
		pb.add( inString, inKey );
		inNonce1 = pb.addNonce( inKey );
		inNonce2 = pb.addNonceNotCrypt();
		pb.addNotCrypt( inByte );
		pb.addNotCrypt( inLong );
		pb.addNotCrypt( inString );
		byte[] resPak = pb.getPaket();
		
		
		IPacketReader pr = new PacketReader( resPak );
		assertEquals( pr.getMessageType(), Message.M_FIRST );
		assertEquals( pr.readData( inKey ), inByte );
		assertEquals( pr.readLong( inKey ), inLong );
		assertEquals( pr.readString( inKey ), inString );
		assertEquals( pr.readNonce( inKey ), inNonce1 );
		assertEquals( pr.readNonceNonCrypt(), inNonce2 );
		assertEquals( pr.readDataNonCrypt(), inByte );
		assertEquals( pr.readLongNonCrypt(), inLong );
		assertEquals( pr.readStringNonCrypt(), inString );
		}
		catch( Exception e ){
			assertTrue( false );		
		}
	}

	public void test2() throws Exception
	{	
		try{
		IPacketBuilder pb = new PacketBuilder( Message.M_LAST );
		byte[] inByte = new byte[128];
		long inLong = 545094534543L;
		long inNonce1;
		String inString = "Dies ist ein Test.";
		byte[] inKey = new byte[8];
		Random rnd = new Random();

		for( int i = 0 ; i < 128 ; i++ )
			inByte[i] = (byte)rnd.nextInt(255);

		for( int i = 0 ; i < 8 ; i++ )
			inKey[i] = (byte)rnd.nextInt(255);

		byte[] inWrongKey = new byte[8];
		for( int i = 0 ; i < 8 ; i++ )
			inWrongKey[i] = inKey[i];
		
		inWrongKey[0] = (byte)(inWrongKey[0] + 23);
		inWrongKey[1] = (byte)(inWrongKey[1] + 11);
			
		pb.add( inByte, inKey );
		pb.add( inLong, inKey );
		pb.add( inString, inKey );
		inNonce1 = pb.addNonce( inKey );
		byte[] resPak = pb.getPaket();
		
		IPacketReader pr = new PacketReader( resPak );
		
		assertEquals( pr.getMessageType(), Message.M_LAST );

		boolean exceptionThrown = false;

		try
		{		
			byte[] read = pr.readData( inWrongKey ); 
			assertEquals( read, inByte );
			fail("Hier htte eine PasswortFalschException auftreten mssen");
		}
		catch (PasswortFalschException e) { }
		
		try{ assertEquals( pr.readLong( inWrongKey ), inLong ); }
		catch( PasswortFalschException e ){ exceptionThrown = true; }
		assertTrue( exceptionThrown );
		exceptionThrown = false;

		try{ assertEquals( pr.readString( inWrongKey ), inString ); }
		catch( PasswortFalschException e ){ exceptionThrown = true; }
		assertTrue( exceptionThrown );
		exceptionThrown = false;

		try{ assertEquals( pr.readNonce( inWrongKey ), inNonce1 ); }
		catch( PasswortFalschException e ){ exceptionThrown = true; }
		assertTrue( exceptionThrown );
		exceptionThrown = false;
		}
		catch( Exception e ){
			assertTrue( false );		
		}
	
	}

	public void output( String text )
	{
		if(output)
			System.out.println(text);				
	}
	
	public static void assertEquals(byte[] array1, byte[] array2)
	{
		assertEquals(array1.length, array2.length);
		for(int i=0; i<array1.length; i++)
		{
			assertEquals(array1[i],array2[i]);
		}
	}

}