/*
 * Created on 20.10.2003
 */
package kerberos.tests.sockets;

import java.io.IOException;
import java.net.Socket;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import kerberos.exceptions.FactoryDestroyedException;
import kerberos.exceptions.KerberosException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UnbekannterParticipalException;
import kerberos.messages.MessageBuilder;
import kerberos.net.KerberosClientSocketFactory;
import kerberos.net.KerberosFactory;
import kerberos.net.KerberosSocket;
import kerberos.net.Passwort;
import kerberos.packets.PacketException;
import kerberos.server.KDCServer;
import util.NetUtil;

/**
 * siehe "SharedDocs/Kerberos/Kerberos - Zu testende Flle.rtf"
 * @author Stephan Koops
 */
public class GanzVieleTests extends TestCase
{
	protected int PORT = 2000;
	
	/** Wenn dieser Wert auf true gesetzt wird, dann werden auch Nicht-Messages gesendet, 
	 *  sondern "irgendwelche" Byte-Arrays */
	public static final boolean mistSenden = true;
	
	/** Wenn dieser Wert auf true gesetzt wird, dann werden auch nicht zu erwartende, 
	 *  aber gltige Messages gesendet */
	public static final boolean ungltigesSenden = true;
	
	public GanzVieleTests(String name)
	{
		super(name);
		System.out.println("Der KDC muss laufen !!!");
	}

	public static Test suite()
	{
		return new TestSuite(GanzVieleTests.class);
	}

	public void setUp()
	{
	}
	
	protected KerberosClientSocketFactory createClientSocketFactory() throws KerberosException, IOException
	{
		return KerberosFactory.clientSocketFactory("testuser", new Passwort("test"),"localhost");
	}
	
	public void testUserOkPasswortOK() throws KerberosException, IOException
	{
		createClientSocketFactory();
	}
	
	public void testFactoryDestroyed() throws KerberosException, IOException
	{
		KerberosClientSocketFactory socketFactory = createClientSocketFactory();
		socketFactory.destroy();
		try
		{
			socketFactory.connect("abc",4711,"def");
			fail("Hier htte eine FactoryDestroyedException auftreten mssen");
		}
		catch (FactoryDestroyedException e)
		{
			System.out.println("Wunderbar. Es wurde erkannt, dass die Factory zerstrt wurde");
		}
	}

	public void testUserOkPasswortFalsch() throws KerberosException, IOException
	{
		try
		{
			KerberosFactory.clientSocketFactory("testuser",new Passwort("falschesPasswort"),"localhost");
			fail("Hier htte eine PasswortFalschException auftreten mssen");
		}
		catch (PasswortFalschException e)
		{
		}
	}

	public void testUserFalscherPasswortOK() throws KerberosException, IOException
	{
		try
		{
			KerberosFactory.clientSocketFactory("meier",new Passwort("geheim"),"localhost");
			fail("Hier htte eine PasswortFalschException auftreten mssen");
		}
		catch (UnbekannterParticipalException e)
		{
		}
	}

	public void testUserFalschPasswortFalsch() throws KerberosException, IOException
	{
		try
		{
			KerberosFactory.clientSocketFactory("meier",new Passwort("falsch"),"localhost");
			fail("Hier htte eine PasswortFalschException auftreten mssen");
		}
		catch (UnbekannterParticipalException e)
		{
		}
	}

	public void testAlterNonceWertKDC() throws KerberosException, IOException
	{
		// TODO Versuch, dem KDC ein Paket mit einem alten nonce-Wert einzuspielen  (Ticket Wiedereinspielung)
	}

	public void testAlterNonceWertZieldienst() throws KerberosException, IOException
	{
		// TODO Versuch, ein Paket mit einem alten nonce-Wert einzuspielen  (Ticket Wiedereinspielung)
	}

	public void testIrgendwasKDC() throws KerberosException, IOException
	{
		// TODO dem KDC irgendwelche Byte-Arrays einspielen
	}

	public void testIrgendwasZieldienst() throws KerberosException, IOException
	{
		if(!mistSenden)
			return;
		KerberosClientSocketFactory socketFactory = createClientSocketFactory();
		KerberosSocket kerbSocket1 = socketFactory.connect("localhost", PORT, "testserver");
		kerbSocket1.printRead("!!!! Hier msste gleich eine Exceptions geworfen werden. Es werden irgendwelche Daten gesendet (testIrgendwasZieldienst");
		kerbSocket1.print("disconnect");
		kerbSocket1.close();
		Socket tcpSocket = new Socket("localhost", PORT);
		tcpSocket.getOutputStream().write("khglshjkhbAFOVCJBVSD".getBytes());
		System.out.println("GanzVieleTest.testIrgendwasZieldienst: Wenn es gleich nicht weitergeht, dann gabs einen Fehler, weil der KerberosServerSocket die neue Verbindung nicht annimmt, weil er mit einer anderen, bewusst nicht korrekt aufgebauten Verbindung beschftigt ist (testIrgendwasZieldienst)");
		KerberosSocket kerbSocket = socketFactory.connect("localhost", PORT, "testserver");
		kerbSocket.print("disconnect");
		kerbSocket.close();
		tcpSocket.close();
		System.out.println("GanzVieleTest.testIrgendwasZieldienst: ging weiter");
	}

	public void testArrayZuLangKDC() throws KerberosException, IOException
	{
		// TODO dem KDC irgendwelche Byte-Arrays einspielen
	}

	public void testArrayZuLangZieldienst() throws KerberosException, IOException
	{
		if(!mistSenden)
			return;
		KerberosClientSocketFactory socketFactory = createClientSocketFactory();
		KerberosSocket kerbSocket1 = socketFactory.connect("localhost", 2000, "testserver");
		kerbSocket1.printRead("!!!! Hier msste gleich eine Exceptions geworfen werden, dass das Array zu lang war (testArrayZuLangZieldienst)");
		kerbSocket1.print("disconnect");
		kerbSocket1.close();
		Socket tcpSocket = new Socket("localhost", 2000);
		NetUtil.writeArray(tcpSocket ,"khglshjkhbAFOVCJBVSD".getBytes());
		System.out.println("GanzVieleTest.testArrayZuLangZieldienst: Wenn es gleich nicht weitergeht, dann gabs einen Fehler, weil der KerberosServerSocket die neue Verbindung nicht annimmt, weil er mit einer anderen, bewusst nicht korrekt aufgebauten Verbindung beschftigt ist (testIrgendwasZieldienst)");
		KerberosSocket kerbSocket = socketFactory.connect("localhost", 2000, "testserver");
		kerbSocket.print("disconnect");
		kerbSocket.close();
		tcpSocket.close();
		System.out.println("GanzVieleTest.testArrayZuLangZieldienst: ging weiter");
	}

	public void testUngltigeNachrichtKDC() throws KerberosException, IOException, PacketException
	{
		if(!ungltigesSenden)
			return;
		System.out.println("GanzVieleTests.testUngltigeNachrichtKDC: Der KDC msste gleich melden, dass eine unerwartete Nachricht empfangen wurde");
		Socket tcpSocket = new Socket("localhost", KDCServer.PORT);
		byte[] nichtSinnvolleNachricht = MessageBuilder.currentBuilder().buildError(47,11);
		tcpSocket.getOutputStream().write(nichtSinnvolleNachricht);
		tcpSocket.getOutputStream().flush();
		tcpSocket.close();
		System.out.println("GanzVieleTests.testUngltigeNachrichtKDC: Jetzt msste die Meldung beim KDC da sein");
		createClientSocketFactory();
	}

	public void testUngltigeNachrichtZieldienst() throws KerberosException, IOException, PacketException
	{
		if(!ungltigesSenden)
			return;
		KerberosClientSocketFactory socketFactory = createClientSocketFactory();
		KerberosSocket kerbSocket1 = socketFactory.connect("localhost", 2000, "testserver");
		kerbSocket1.printRead("!!!! Hier msste gleich eine Exceptions geworfen werden, dass eine unerwartete Nachricht gekommen ist (testUngltigeNachrichtZieldienst)");
		kerbSocket1.print("disconnect");
		kerbSocket1.close();
		Socket tcpSocket = new Socket("localhost", 2000);
		byte[] nichtSinnvolleNachricht = MessageBuilder.currentBuilder().buildError(47,11);
		tcpSocket.getOutputStream().write(nichtSinnvolleNachricht);
		tcpSocket.close();
		socketFactory.destroy();
		createClientSocketFactory().connect("localhost", 2000, "testserver");
	}

	public void testAbgelaufenesTicketKDC() throws KerberosException, IOException
	{
		// TODO? Ablauf der Tickets (Lebensdauer beschrnken auf z.B. 5min)
	}

	public void testAbgelaufenesTicketZieldienst() throws KerberosException, IOException
	{
		// TODO Ablauf der Tickets (Lebensdauer beschrnken auf z.B. 5min)
	}

	public void testMehrereServer() throws KerberosException, IOException
	{
		// TODO einmal Anmelden bei Kerberos und dann sich mit mehreren Servern verbinden, ohne dass man sich neu anmelden muss.
	}	
}