/*
 * Created on 20.10.2003
 */
package kerberos.tests.sockets;

import java.io.IOException;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Test;
import kerberos.tests.TestCase;
import junit.framework.TestSuite;
import kerberos.net.KerberosClientSocketFactory;
import kerberos.net.KerberosServerSocket;
import kerberos.net.KerberosSocket;
import kerberos.net.KerberosClientSocketFactoryImpl;
import kerberos.net.KerberosServerSocketImpl;
import kerberos.net.Passwort;

/**
 * @author Stephan Koops
 */
public class KerberosSocketTest extends TestCase
{
	public List written = new ArrayList();
	public boolean output = true;
	
	public int PORT = 2001;
	
	private Exception exception = null;

	public KerberosSocketTest(String name)
	{
		super(name);
	}

	/**
	 * Um die Klasse als Anwendung zu starten
	 */
	protected KerberosSocketTest()
	{
		super();
		this.output = true;
	}

	public static Test suite()
	{
		return new TestSuite(KerberosSocketTest.class);
	}

	public void setUp()
	{
	}

	public static void main(String[] args) throws Exception
	{
		new KerberosSocketTest().test1();
	}

	class Sender extends Thread
	{
		private KerberosSocketTest testcase;
		 
		public Sender(KerberosSocketTest testCase)
		{
			this.testcase = testCase;
		}
		
		public void run()
		{
			try
			{
				Thread.sleep(500);
				KerberosClientSocketFactory socketFactory = new KerberosClientSocketFactoryImpl("testuser", new Passwort("test"),"localhost");
				KerberosSocket socket = socketFactory.connect("localhost",PORT,"testserver");
				toSocket(socket,"Hallo Duda djflwdjhekf".getBytes());
				fromSocketByteArray(socket);
				toSocket(socket,"Hallo Duda djflwdjhekf".getBytes());
				fromSocketByteArray(socket);
				toSocket(socket,"Hallo Duda djflwdjhekf");
				fromSocketByteArray(socket);
				toSocket(socket,"wjnqwuwibgh twfwggvvarc".getBytes());
				fromSocketByteArray(socket);
				toSocket(socket,"Hallo Duda djflwdjhekf".getBytes());
				// In der Methode test1() muss genausooft empfangen und gesendet werden wie hier
				//Thread.sleep(5000);
				socket.close();
				if(output)
					System.out.println("write  beendet");
			}
			catch (Exception e)
			{
				System.err.print("KerberosSocketTests.Sender: "+e.getClass().getName()+": ");
				if (e.getMessage() != null)
					System.err.println(e.getMessage());
				if (e.getCause() != null)
					System.err.println(e.getCause());
				else if (e.getMessage() == null) // beide null
					System.err.println("Die Meldung und der Grund sind null: e="+e);
				testcase.merkeException(e);
				System.out.println("KerberosSocketTests.Sender: Der Sender hat mit einer Excepion abgebrochen");
				try
				{
					Socket socket = new Socket("localhost",PORT); // ggf offenes ServerSocket.accept() beenden
					socket.close();
				}
				catch (IOException e1) { }
			}
		}
	}

	public void merkeException(Exception exception)
	{
		this.exception = new Exception("aus anderem Thread: ",exception);
	}

	public void test1() throws Exception
	{
		KerberosServerSocket serverSocket = null;
		try
		{
			System.out.println("Dieser Test setzt vorraus, dass der KDC auf localhost luft und den Participal testuser und testserver kennt");
			Thread client = new Sender(this);
			client.start();
			
			serverSocket = new KerberosServerSocketImpl(PORT, new Passwort("geheim"));
			KerberosSocket socket = serverSocket.accept();
			// Hier muss genausooft empfangen und gesendet werden, wie in der internen Klasse Sender
			fromSocketByteArray(socket);
			toSocket(socket, new byte[0]);
			fromSocketByteArray(socket);
			toSocket(socket, new byte[0]);
			fromSocketByteArray(socket);
			toSocket(socket, new byte[0]);
			fromSocketByteArray(socket);
			toSocket(socket, new byte[0]);
			fromSocketByteArray(socket);
			toSocket(socket, new byte[0]);
			socket.close();
			serverSocket.close();
			if(output)
				System.out.println(" READ beendet");
			Thread currentThread = Thread.currentThread();
			int prio = currentThread.getPriority();
			currentThread.setPriority(Thread.MIN_PRIORITY);
			if (client.isAlive())
				System.out.println("KerberosSocketTest.test1() Der Thread wartet noch auf den Client");
			while(client.isAlive())
			{
				Thread.yield(); // CPU abgeben
			}
			currentThread.setPriority(prio);
			if(this.exception != null)
				throw this.exception;
		}
		catch (Exception e)
		{
			if (serverSocket != null)
				serverSocket.close();
			throw e;
		}
	}

	public void fromSocketByteArray(KerberosSocket socket) throws Exception
	{
		byte[] byteArray = socket.read();				
		if(output)
			System.out.print("  READ, byte[]: ");
		printBytes(byteArray);
		Object geschriebenO = written.get(0);
		byte[] geschrieben;
		if(geschriebenO instanceof byte[])
			geschrieben = (byte[])geschriebenO;
		else 
			geschrieben = ((String)geschriebenO).getBytes(); 
		assertEquals(geschrieben,byteArray);
		written.remove(0);
		if(output)
			System.out.println();				
	}

	/**
	 * sendet das Byte-Array durch den Socket
	 * @param socket
	 * @param byteArray
	 * @throws Exception
	 */
	private void toSocket(KerberosSocket socket, byte[] byteArray) throws Exception
	{
		if(output)
			System.out.print("write,  byte[]: ");
		printBytes(byteArray);
		if(output)
			System.out.println();
		written.add(byteArray);
		socket.print(byteArray);
		Thread.sleep(100);
	}
	
	/**
	 * sendet das Byte-Array durch den Socket
	 * @param socket
	 * @param byteArray
	 * @throws Exception
	 */
	private void toSocket(KerberosSocket socket, String byteArray) throws Exception
	{
		if(output)
			System.out.print("write,  byte[]: ");
		printBytes(byteArray);
		if(output)
			System.out.println();
		written.add(byteArray);
		socket.print(byteArray);
		Thread.sleep(100);
	}
	
	private void printBytes(byte[] bytes)
	{
		if(output)
		{
			Passwort.toZahl(bytes);
		}
	}
	private void printBytes(String bytes)
	{
		if(output)
		{
			System.out.print(bytes);
		}
	}
}