/*
 * Created on 17.11.2003
 */
package kerberos.tutorial;

import util.logger.SystemOutLogger;
import kerberos.net.KerberosClientSocketFactory;
import kerberos.net.KerberosFactory;
import kerberos.net.KerberosSocket;
import kerberos.net.Passwort;

/**
 * Das Package kerberos.tutorial.TutorialZieldienst ist ein Mini-Tutorial und soll zeigen, wie die 
 * Kommunikation mit den Kerberos-Klassen funktioniert. Die weiteren allgemeinen Erluterungen finden
 * sich in der Klasse TutorialZieldienst
 * @see kerberos.tutorial.TutorialZieldienst  
 * @author Stephan Koops
 */
public class TutorialClient
{
	public static void main(String[] args) throws Throwable
	{
		// Passwort-Objekt erzeugen
		Passwort passwort = new Passwort("streng geheim"); // es ist nicht so gut, diesen Konstruktor zu benutzen. Siehe Anmerkung im Javadoc des Konstruktors 

		// Die gleich erzeugte Socket-Factory speichert die bentigten Schlssel und authentifiziert sich beim
		// KDC und besorgt sich das von dort das Ticket-Granting-Ticket 
		KerberosClientSocketFactory socketFactory = KerberosFactory.clientSocketFactory("usernamen", passwort, "kdc.organisation.org");
		
		// Hier kann ein anderer Logger angegeben werden. Normalerweise wird nicht geloggt.
		// @see kerberos.net.Logger
		socketFactory.setLogger(new SystemOutLogger());
		
		// Verbindung zum Server aufbauen: Anfrage an den TicketGranting-Service
		KerberosSocket socket = socketFactory.connect("hostname", TutorialZieldienst.PORT, "TutorialZieldienst");

		// Applikation: Nutzdaten schreiben		
		socket.print("Hallo !");
		// lesen
		String antwort1 = socket.readString();
		System.out.println("Antwort1: "+antwort1);

		// schreiben und sofort lesen
		String antwort2 = socket.printRead("Wie gehts !!");
		System.out.println("Antwort2: "+antwort2);
		
		socket.print("ende");
		socket.read(); // Der Server antwortet noch, dass er fertig ist
		
		// Verbindung beenden
		socket.close();
		
		// mit der socketFactory mit Hilfe der Methode connect(..) kann man sich zu beliebig vielen Servern aufzubauen
		
		// ab diesem Aufruf ist die SocketFactory unbenutzbar
		socketFactory.destroy();
	}
}
