/*
 * Created on 12.01.2004
 */
package kerberos.tutorial;

import java.io.IOException;

import kerberos.exceptions.KerberosException;
import kerberos.net.KerberosSocket;

/**
 * Diese Klasse stellt die Thread-Funktionalitt fr den Tutrorial-Server bereit, 
 * die fr jeden einzelnen Client bentigt wird.
 * Er antwortet mit dem bergebenen Text, fgt
 * aber noch ein bichen hinzu.
 * Beendet wird der Thread, indem man den ihm durch den 
 * Socket den String "ende" sendet.
 * @see kerberos.tutorial.TutorialZieldienst
 * @see #tutorial()
 * @author Stephan Koops
 */
public class TutorialDienstThread extends Thread
{
	private KerberosSocket socket;
	
	public TutorialDienstThread(KerberosSocket socket)
	{
		this.socket = socket;
	}

	/**
	 * Diese Methode wird vom Thread gestartet.<br>
	 * Die didaktische wertvolle Methode ist die Methode <a href="#tutorial">tutorial()</a>.
	 * Die Methode run() macht sonst nur Exception-Handling
	 * @see java.lang.Runnable#run()
	 * @see #tutorial()
	 */
	public void run()
	{
		try
		{
			tutorial(); // Diese Methode sollte man sich im Tutorial ansehen
		}
		catch (KerberosException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if(!socket.isClosed())
				{
					socket.close();
					System.out.println("Socket geschlossen.");
				}
			}
			catch (IOException e1)
			{
				System.err.println("Der Socket konnte nicht geschlossen werden.");
				e1.printStackTrace();
			}
		}
	}
	
	/**
	 * In dieser Methode wird die eigentliche Arbeit gemacht.
	 * Der Rest in der Methode run() ist Exception-Handling.
	 * @throws KerberosException Alle vom Kerberos-Paket geworfenen Exceptions 
	 * (auer der IOException) sind von der KerberosException abgeleitet.
	 * Wenn es also um eine pauschale Fehlerbehandlung geht, dann kann braucht
	 * der catch-Block nur die KerberosException und die IOException abfangen. 
	 * @throws IOException
	 */
	private void tutorial() throws KerberosException, IOException
	{
		System.out.println("Der Benutzer "+socket.getUsername()+" hat eine Verbindung aufgebaut");
		String daten;
		for(;;) // Beendet wird die Schleife mit return
		{
			daten = this.socket.readString();
			if(daten.equalsIgnoreCase("ende") || daten.equalsIgnoreCase("\"ende\"") || daten.equalsIgnoreCase("disconnect"))
			{
				socket.print("Tsch");
				socket.close();
				return;
			}
			System.out.println("Ich habe \""+daten+"\" empfangen");
			this.socket.print("Sehr wohl. Ich habe verstanden: "+daten+"\nEnde mit dem String \"ende\"");
			// auerdem gibt es noch die Methoden printRead(String) und printRead(byte[]), welche eine Nachricht
			// senden und dann gleich die Antwort abwarten
		}
	}
}
