package kerberos.tutorial;

import util.logger.SystemOutLogger;
import kerberos.net.KerberosFactory;
import kerberos.net.KerberosServerSocket;
import kerberos.net.KerberosSocket;
import kerberos.net.Passwort;

/**
 * <p>In diesem Mini-Tutorial wird zusammen mit der Klasse kerberos.tutorial.TutorialClient und 
 * kerberos.tutorial.TutorialDienstThread erklrt,
 * wie die Kommunikation mit den Kerberos-Klassen funktioniert.</p>
 * <p>Fr dieses Tutorial ist es exrem vorteilhaft, wenn man wei, wie java.net.ServerSocket und java.net.Socket
 * funktionieren. Wie Kerberos an sich funktioniert, wird hier ebenfalls nicht erklrt, sondern nur, wie
 * man diese Implementation nutzt.</p>
 * <p>Diese Klasse stellt einen Server dar, der Anfragen von Clients entgegen nimmt, die bertragenen
 * Bytes als String anzeigt und dann ein paar Zeichen an den anfragenden Client zurckschickt.</p>
 * <p>Um ber Kerberos zu kommunizieren, erzeugt man auf der <b>Serverseite</b> einen 
 * <a href="../net/IKerberosServerSocket.html">kerberos.net.IKerberosServerSocket.</a>.
 * Dieser wartet wie ein normaler ServerSocket auf Verbindungen 
 * (Methode <a href="../net/IKerberosServerSocket.html#accept()">accept()</a>) und gibt 
 * bei der Anfrage eines Clients einen <a href="../net/IKerberosSocket.html">IKerberosSocket</a>
 * zurck, entsprechend einem normalen java.net.Socket.<br>
 * Auf der Client-Seite wird eine kerberos.net.IKerberosClientSocketFactory erzeugt. Diese speichert z.B. die
 * Schlssel zum Verschlsseln der Verbindung zum Ticket-Granting-Service des KDC. Mit Hilfe der Methode
 * IKerberosClientSocketFactory.connect(String host, int ziel, String Zieldienst) wird eine Verbindung zu
 * dem angegebenen Zieldienst auf dem angegebenen Host aufgebaut. Die Verbindung wird dabei verschlsselt und
 * authentifiziert durchgefhrt.</p>
 * Die Erzeugung vom kerberos.net.IKerberosServerSocket und der kerberos.net.KerberosClientSocketFactory
 * kann von der kerberos.net.KerberosFactory bernommen werden.
 * <p>Was die einzelnen Methoden genau bewirken, wird in den jeweiligen javadocs erklrt.</p> 
 * 
 * @see kerberos.net.KerberosServerSocket
 * @see kerberos.net.KerberosSocket
 * @see kerberos.net.KerberosClientSocketFactory
 * @see kerberos.net.KerberosFactory
 * @see kerberos.tutorial.TutorialDienstThread
 * @see kerberos.tutorial.TutorialClient
 * @see java.net.ServerSocket
 * @see java.net.Socket
 * @author Stephan Koops
 */
public class TutorialZieldienst
{
	public static final int PORT = 4711; // Port des Zieldienstes
	
	public static void main(String[] args) throws Exception
	{
		Passwort passwortDesZieldienstesBeimKDC = new Passwort("streng geheim");
		
		// Einen KerberosServerSocket erzeugen, welcher auf die Verbindungen der Clients wartet.
		KerberosServerSocket serverSocket = KerberosFactory.serverSocket(PORT, passwortDesZieldienstesBeimKDC);
		
		// Hier kann ein anderer Logger angegeben werden. Normalerweise wird nicht geloggt.
		// @see kerberos.net.Logger
		serverSocket.setLogger(new SystemOutLogger());
		
		System.out.println("Der TutorialTestserver wartet auf Verbindungen.");
		System.out.println("Beenden des Programmes ist z.Zt. nur durch gewaltsames Beenden des Prozesses mglich, z.B. den roten Button in Eclipse");

		// Immer auf neue Verbindungen warten
		while (true)
		{
			// Warten, bis ein Client eine Anfrage stellt und diese annehmen.
			KerberosSocket socket = serverSocket.accept();
			
			// Den erzeugten Socket einem neuen Thread bergeben und diesen starten
			Thread thread = new TutorialDienstThread(socket);
			thread.start();
		}
	}
}
