/*
 * Created on 14.01.2004
 */
package util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * Diese Queue blockiert, wenn Objekte abgefragt werden sollen,
 * aber keine mehr da sind, bis neue Objekte hinzugefgt wurden
 * @author Stephan
 */
public class BlockingQueue implements Queue
{
	/** Diese Liste speichert die Elemente dieser Queue */
	protected List elemente = new ArrayList();
	
	/** Diese Semaphore kontrolliert, ob genug Elemente enthalten sind.
	 * Der Counter der Semaphore ist genauso gro wie die Anzahl der enthaltenen Elemente
	 * Wenn die Queue leer ist, dann blockeirt sie beim get() */
	protected Semaphore semaphore = new Semaphore(elemente.size());

	public BlockingQueue(int initialCapacity) 
	{
		this.elemente = new ArrayList(initialCapacity);
	}

	 /**
	  * Constructs a list containing the elements of the specified
	  * collection, in the order they are returned by the collection's
	  * iterator.  The <tt>ArrayList</tt> instance has an initial capacity of
	  * 110% the size of the specified collection.
	  *
	  * @param c the collection whose elements are to be placed into this list.
	  * @throws NullPointerException if the specified collection is null.
	  */
	public BlockingQueue(Collection coll) 
	{
		this(coll.size());
		semaphore = new Semaphore(elemente.size());
		this.addAll(coll);
	}
	
	public BlockingQueue()
	{
	}

	/**
	 * @see util.Queue#size()
	 */
	public synchronized int size()
	{
		return elemente.size();
	}

	/**
	 * @see util.Queue#isEmpty()
	 */
	public synchronized boolean isEmpty()
	{
		return elemente.isEmpty();
	}

	/**
	 * @see util.Queue#add(java.lang.Object)
	 */
	public boolean add(Object o)
	{
		boolean rtn;
		synchronized (this)
		{
			rtn = elemente.add(o);
		}
		//System.out.println("BlockingQueue.add: Element "+o+" hinzugefgt; gleich Semaphore verlassen");
		semaphore.verlassen();
		//System.out.println("BlockingQueue.add: Semaphore soeben verlassen");
		return rtn;
	}

	/**
	 * @see util.Queue#get()
	 */
	public Object get()
	{
		//System.out.println("BlockingQueue.get: gleich Semaphore betreten");
		semaphore.betreten();
		//System.out.println("BlockingQueue.get: Semaphore betreten, gleich Element "+elemente.get(0)+" zurckgeben");
		synchronized (this)
		{
			return elemente.remove(0);
		}
	}

	/**
	 * @see util.Queue#addAll(java.util.Collection)
	 */
	public boolean addAll(Collection coll)
	{
		int size = size();
		Iterator iter = coll.iterator();
		while (iter.hasNext())
		{
			this.add(iter.next());
		}
		return size == size();
	}

	/**
	 * @see util.Queue#clear()
	 */
	public synchronized void clear()
	{
		elemente.clear();
	}
	
	public synchronized int hashCode()
	{
		return elemente.hashCode();
	}
	
	/*public boolean equals(Object queue)
	{
		if (queue instanceof Queue)
		{
			return equals((Queue)queue);
		}
		else
		{
			return false;
		}
	}
	
	public boolean equals(Queue queue)
	{
		return 
	} //*/
}
