/*
 * Created on 20.10.2003
 */
package util.tests;

import util.BlockingQueue;
import util.Queue;
import junit.framework.Test;
import junit.framework.TestSuite;
import kerberos.tests.TestCase;

/**
 * @author Stephan Koops
 */
public class BlockingQueueTests extends TestCase
{
	/** Eine am Anfang leere Queue */
	protected Queue queue0;
	
	/** Diese Methode wird auf true gesetzt, wenn das Warten in testBlocking fertig ist */
	protected Boolean waitingReady;
	
	public BlockingQueueTests(String name)
	{
		super(name);
	}

	public static Test suite()
	{
		return new TestSuite(BlockingQueueTests.class);
	}

	public void setUp()
	{
		this.queue0 = new BlockingQueue();
		this.waitingReady = Boolean.FALSE;
	}
	
	public void testSize()
	{
		assertEquals(this.queue0.size(), 0);
		this.queue0.add("kh");
		assertEquals(this.queue0.size(), 1);
		Object obj = this.queue0.get();
		assertEquals(this.queue0.size(), 0);
		assertTrue(obj instanceof String);
	}
	
	public void testBlocking()
	{
		new Thread()
		{
			public void run()
			{
				BlockingQueueTests.sleep(1000);
				System.out.println(">abc< hinzufgen");
				queue0.add("abc");
				System.out.println(">abc< hinzugefgt");
				synchronized(waitingReady)
				{
					waitingReady = Boolean.TRUE;
				}
			}
		}.start();
		synchronized(waitingReady)
		{
			assertTrue(waitingReady == Boolean.FALSE);
			if (queue0.size() != 0)
				throw new RuntimeException("Dieser Test hat leider nicht geklappt, da der Test zeitlich nicht zu synchronisieren ist");
		}
		System.out.println("Warten, bis ein Element da ist");
		queue0.get();
		Thread.yield();
		System.out.println("Es war ein Element da");
		assertEquals(queue0.size(), 0);
	}
	
	/**
	 * Schlft die angegebene Zeit. Bei einer InterruptedException wird das 
	 * Schlafen abgebrochen und die Exception ignoriert.
	 * @param millis Schlafen-Dauer in Millisekunden
	 * @see Thread#sleep(long)
	 */
	public static void sleep(long millis)
	{
		try
		{
			Thread.sleep(millis);
		}
		catch (InterruptedException e)
		{
		}
	}
}